defmodule AdvertisementListContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Properties.Property
  alias PropTrackr.Accounts.User

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists such advertisements$/, fn state, %{table_data: table} ->
    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> Property.changeset(%Property{}, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I am not logged in$/, fn state ->
    navigate_to("/logout")
    {:ok, state}
  end

  when_ ~r/^I visit the property advertisement list page$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  then_ ~r/^I should see the list of all properties$/, fn state ->
    advertisements = state[:advertisements]
    advertisements |> Enum.each(fn advertisement ->
      assert visible_in_page? ~r/#{advertisement.title}/
      assert visible_in_page? ~r/#{advertisement.description}/
      assert visible_in_page? ~r/#{advertisement.location}/
      assert visible_in_page? ~r/#{advertisement.price}/
      assert visible_in_page? ~r/#{advertisement.room_count}/
      assert visible_in_page? ~r/#{advertisement.area}/
    end)

    {:ok, state}
  end

  then_ ~r/^I should see a message that there is no advertisements$/, fn state ->
    assert visible_in_page? ~r/No advertisements at the moment/
    {:ok, state}
  end



  # FR-17 - Display Property Owner Contact Details

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table}->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  and_ ~r/^I want to insert the following data$/, fn state, %{table_data: table} ->
    data = List.first(table)
    {
      :ok,
      state
      |> Map.put(:data, data)
    }
  end

  and_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:email], state[:password])
    {:ok, state}
  end

  and_ ~r/^I want to create a new property advertisement$/, fn state ->
    navigate_to("/")
    click({:id, "add_advertisement"})
    {:ok, state}
  end

  and_ ~r/^I fill in the property advertisement form with valid data$/, fn state ->
    fill_field({:id, "title"}, state[:data][:title])
    fill_field({:id, "description"}, state[:data][:description])
    select_dropdown("type", state[:data][:type])
    select_dropdown("property_type", state[:data][:property_type])
    fill_field({:id, "location"}, state[:data][:location])
    fill_field({:id, "area"}, state[:data][:area])
    fill_field({:id, "room_count"}, state[:data][:room_count])
    fill_field({:id, "floor"}, state[:data][:floor])
    fill_field({:id, "floor_count"}, state[:data][:floor_count])
    fill_field({:id, "price"}, state[:data][:price])

    file_input = find_element(:id, "property_photos")

    execute_script("""
      (function() {
        const input = document.getElementById('property_photos');
        const files = [
          new File(['test content'], 'test_photo_1.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_2.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_3.jpg', { type: 'image/jpeg' })
        ];
        const dataTransfer = new DataTransfer();
        files.forEach(file => dataTransfer.items.add(file));
        input.files = dataTransfer.files;
        input.dispatchEvent(new Event('change', { bubbles: true }));
      })();
    """)

    :timer.sleep(100)

    {:ok, state}
  end

  when_ ~r/^I click submit$/, fn state ->
    click({:id, "submit_button"})
    {:ok, state}
  end

  then_ ~r/^I should see a success message$/, fn state ->
    assert visible_in_page? ~r/Property created successfully!/
    {:ok, state}
  end

  and_ ~r/^I log out$/, fn state ->
    navigate_to("/logout")
    {:ok, state}
  end

  and_ ~r/^I go to homepage$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  when_ ~r/^I click the View more button on "(?<argument_one>[^"]+)" advertisement$/, fn state, %{argument_one: argument_one} ->
    button = find_element(:css, "[data-title='#{argument_one}']")
    click(button)

    {:ok, state}
  end

  then_ ~r/^I should see the contact details of the owner:$/, fn state, %{table_data: table} ->
    expected_owner = List.first(table)

    assert visible_in_page?(~r/#{expected_owner["owner_name"]}/)
    assert visible_in_page?(~r/#{expected_owner["owner_surname"]}/)
    assert visible_in_page?(~r/#{expected_owner["owner_phone"]}/)
    assert visible_in_page?(~r/#{expected_owner["owner_email"]}/)

    {:ok, state}
  end

  and_ ~r/^there exists such advertisements new$/, fn state, %{ table_data: table } ->
    owner = Repo.get_by(User, email: state[:email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    countries = Enum.map(advertisements, fn advertisement -> advertisement.location end)
                |> Enum.map(fn location -> String.split(location, ",") end)
                |> Enum.map(fn location -> List.last(location) end)
                |> Enum.map(fn location -> String.trim(location) end)
                |> Enum.uniq()

    areas = Enum.map(advertisements, fn advertisement -> advertisement.location end)
            |> Enum.map(fn location -> String.split(location, ",") end)
            |> Enum.map(fn location -> List.first(location) end)
            |> Enum.map(fn location -> String.trim(location) end)
            |> Enum.uniq()

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
      |> Map.put(:countries, countries)
      |> Map.put(:areas, areas)
    }
  end

  and_ ~r/^I click the Calculate Price button$/, fn state ->
    click({:id, "calculate_button"})
    {:ok, state}
  end

  then_ ~r/^I should see the calculated price displayed and it should be 450$/, fn state ->
    calculated_price_element = find_element(:id, "calculated_price")
    calculated_price_value = attribute_value(calculated_price_element, "value")
    assert calculated_price_value == "450"

    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

  # from advertisement_creation_context.exs
  # https://stackoverflow.com/a/49861811
  defp select_dropdown(drop_down_id, option) do
    find_element(:css, "##{drop_down_id} option[value='#{option}']") |> click()
  end

end
