defmodule DarkModeContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Properties.Property
  alias PropTrackr.Accounts.User
  alias PropTrackr.Repo

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details ->
      # Handle is_dark_mode if present
      user_details = case Map.get(user_details, :is_dark_mode) do
        nil -> user_details
        value when is_binary(value) -> Map.put(user_details, :is_dark_mode, value == "true")
        _ -> user_details
      end
      User.changeset(%User{}, user_details)
    end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    user = List.last(table)
    {
      :ok,
      state
      |> Map.put(:user_email, user[:email])
      |> Map.put(:user_password, user[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{ table_data: table } ->
    owner = Repo.get_by(User, email: state[:user_email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  # Navigation steps
  when_ ~r/^I visit the home page$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  given_ ~r/^I am on the home page$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  when_ ~r/^I click the theme toggle button$/, fn state ->
    find_element(:css, "[data-theme-toggle]") |> click()
    :timer.sleep(500)
    {:ok, state}
  end

  given_ ~r/^I have enabled dark mode$/, fn state ->
    navigate_to("/")
    find_element(:css, "[data-theme-toggle]") |> click()
    :timer.sleep(500)
    {:ok, state}
  end

  when_ ~r/^I navigate to the (?<page>[^"]+) page$/, fn state, %{page: page} ->
    path = case page do
      "profile" -> "/me"
      "login" -> "/login"
      "home" -> "/"
    end
    navigate_to(path)
    :timer.sleep(500)
    {:ok, state}
  end

  when_ ~r/^I refresh the page$/, fn state ->
    refresh_page()
    :timer.sleep(500)
    {:ok, state}
  end

  when_ ~r/^I visit the property details page$/, fn state ->
    property = hd(state.advertisements)
    navigate_to("/properties/#{property.reference}")
    :timer.sleep(500)
    {:ok, state}
  end

  # Light mode assertions
  then_ ~r/^I should see the application in light mode$/, fn state ->
    :timer.sleep(500)
    html_classes = find_element(:css, "html") |> attribute_value("class")
    refute String.contains?(html_classes, "dark")
    assert has_classes?("body", ["bg-white", "text-zinc-900"])
    assert element_displayed?(find_element(:css, ".theme-toggle-button .hero-moon-solid"))
    {:ok, state}
  end

  # Dark mode assertions
  then_ ~r/^I should see the application in dark mode$/, fn state ->
    :timer.sleep(500)
    html_classes = find_element(:css, "html") |> attribute_value("class")
    assert String.contains?(html_classes, "dark")
    assert has_classes?("body", ["dark:bg-dark-background", "dark:text-dark-text"])
    assert element_displayed?(find_element(:css, ".theme-toggle-button .hero-sun-solid"))
    {:ok, state}
  end

  then_ ~r/^I should see the theme toggle button shows (?<icon>[^"]+) icon$/, fn state, %{icon: icon} ->
    icon_class = case icon do
      "moon" -> "hero-moon-solid"
      "sun" -> "hero-sun-solid"
    end
    assert element?(:css, ".#{icon_class}")
    {:ok, state}
  end

  then_ ~r/^the background should be dark$/, fn state ->
    :timer.sleep(500)
    assert has_classes?("body", ["dark:bg-dark-background"])
    {:ok, state}
  end

  and_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:user_email], state[:user_password])
    :timer.sleep(500)
    {:ok, state}
  end

  and_ ~r/^the theme toggle should show the moon icon$/, fn state ->
    moon_icon = find_element(:css, ".theme-toggle-button .hero-moon-solid")
    assert element_displayed?(moon_icon)
    {:ok, state}
  end

  and_ ~r/^the theme toggle should show the sun icon$/, fn state ->
    sun_icon = find_element(:css, ".theme-toggle-button .hero-sun-solid")
    assert element_displayed?(sun_icon)
    {:ok, state}
  end

  then_ ~r/^the text should be light colored$/, fn state ->
    :timer.sleep(500)
    assert has_classes?("body", ["dark:text-dark-text"])  # Using the actual class from your HTML
    {:ok, state}
  end

  then_ ~r/^I should see the (?<page>[^"]+) page in dark mode$/, fn state, %{page: page} ->
    assert get_css_variable("--color-background") == "#0f172a"
    {:ok, state}
  end

  then_ ~r/^I should see dark mode colors for:$/, fn state, %{table_data: table} ->
    variables = %{
      "Background" => "--color-background",
      "Text" => "--color-text",
      "Cards" => "--color-card",
      "Buttons" => "--color-primary"
    }

    {:ok, Map.put(state, :colors,
      Enum.reduce(table, %{}, fn row, acc ->
        css_var = variables[row["Element"]]
        value = get_css_variable(css_var)
        Map.put(acc, row["Element"], value)
      end))}
  end

  defp setup_session(email, password) do
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

  defp get_css_variable(variable_name) do
    script = """
    var root = document.documentElement;
    var theme = root.getAttribute('data-theme');
    var style = getComputedStyle(root);
    var value = style.getPropertyValue('#{variable_name}').trim();
    console.log('Theme:', theme, 'Variable:', '#{variable_name}', 'Value:', value);
    return value;
    """
    execute_script(script)
  end

  defp get_element_classes(selector) do
    find_element(:css, selector) |> attribute_value("class")
  end

  defp has_classes?(selector, classes) when is_list(classes) do
    element_classes = get_element_classes(selector)
    Enum.all?(classes, &String.contains?(element_classes, &1))
  end
 end
