defmodule FavoriteProperties do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    random_user = List.first(table)
    owner = List.last(table)

    {
      :ok,
      state
      |> Map.put(:random_email, random_user[:email])
      |> Map.put(:random_password, random_user[:password])
      |> Map.put(:owner_email, owner[:email])
      |> Map.put(:owner_password, owner[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{ table_data: table } ->
    owner = Repo.get_by(User, email: state[:owner_email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I am logged in as random user$/, fn state ->
    setup_session(state[:random_email], state[:random_password])
    {:ok, state}
  end

  and_ ~r/^the property "(?<argument_one>[^"]+)" is in my favorites$/,
  fn state, %{argument_one: argument_one} ->
    current_user = Repo.get_by(User, email: state[:random_email])

    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    favorite_changeset = %Favorite{}
    |> Favorite.changeset(%{
      user_id: current_user.id,
      property_id: property.id
    })

    Repo.insert!(favorite_changeset)

    {:ok, state}
  end

  when_ ~r/^I visit my profile page$/, fn state ->
    navigate_to("/me")
    {:ok, state}
  end

  when_ ~r/^I visit the favorites page$/, fn state ->
    navigate_to("/me/favorites")
    {:ok, state}
  end

  then_ ~r/^I should be redirected to login page$/, fn state ->
    current_path = current_url()
    assert String.contains?(current_path, "/login"),
      "Expected to be redirected to login page but was on #{current_path}"
    {:ok, state}
  end


  and_ ~r/^I click "(?<argument_one>[^"]+)" button$/,
  fn state, %{argument_one: argument_one} ->
    case argument_one do
      "My favorites" ->
        find_element(:id, "my_favorites")
        |> click()

      other_button ->
        button = find_element(:xpath, "//button[contains(text(), '#{other_button}')]")
        click(button)
    end

    :timer.sleep(500)
    {:ok, state}
  end

  and_ ~r/^I click "(?<argument_one>[^"]+)" for "(?<argument_two>[^"]+)"$/,
  fn state, %{argument_one: argument_one, argument_two: argument_two} ->
    button = find_element(
      :xpath,
      "//div[contains(., '#{argument_two}')]//button[contains(., '#{argument_one}')]"
    )
    click(button)

    :timer.sleep(500)

    {:ok, state}
  end

  then_ ~r/^I should see "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    :timer.sleep(100)

    # First try to find the text in flash messages
    flash_visible = case find_all_elements(:css, ".flash, .alert, [role='alert']") do
      [] -> false
      elements ->
        Enum.any?(elements, fn element ->
          element_text = visible_text(element)
          String.contains?(element_text, argument_one)
        end)
    end

    # If not in flash, check the general page content
    page_contains = String.contains?(page_source(), argument_one)

    assert flash_visible || page_contains,
      "Expected to see '#{argument_one}' on the page or in flash messages but it was not found"

    {:ok, state}
  end

  then_ ~r/^I should be on the details page for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    current_url = current_url()
    expected_path = "/properties/#{property.reference}"

    assert String.contains?(current_url, expected_path),
      "Expected to be on property details page #{expected_path} but was on #{current_url}"

    page_text = page_source()
    assert String.contains?(page_text, argument_one),
      "Expected to see property title '#{argument_one}' on the details page"

    {:ok, state}
  end

  and_ ~r/^I should not see "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    page_text = page_source()
    refute String.contains?(page_text, argument_one),
      "Expected NOT to see '#{argument_one}' on the page but it was found"

    {:ok, state}
  end

  when_ ~r/^the property "(?<argument_one>[^"]+)" becomes unavailable$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    property
    |> Property.state_changeset(%{state: :unavailable})
    |> Repo.update!()

    {:ok, state}
  end



  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
end
