defmodule MessagingContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Properties.Property
  alias PropTrackr.Accounts.User
  alias PropTrackr.Messaging.{Message, MessageAssociation}
  import Ecto.Query, only: [from: 2]
  import Ecto.Changeset

  @password "password"

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{table_data: table} ->
    owner = Repo.get_by(User, email: state[:email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^the following messages exist$/, fn state, %{ table_data: table } ->
    # Note: Message is sent by another user
    user = from(
      u in User,
      where: u.email != ^state[:email],
      select: u
    ) |> Repo.one()

    author_user = Repo.get_by(User, email: state[:email])

    messages = table
      |> Enum.map(fn details ->
        Message.changeset(%Message{}, %{
          title: details[:title],
          content: details[:content]
        })
        |> put_change(:user_id, user.id)
      end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    assocs = Enum.map(messages, fn message ->
      Repo.insert(
        MessageAssociation.changeset(%MessageAssociation{}, %{})
        |> put_change(:user_id, user.id)
        |> put_change(:message_id, message.id)
      )

      Repo.insert(
        MessageAssociation.changeset(%MessageAssociation{}, %{})
        |> put_change(:user_id, author_user.id)
        |> put_change(:message_id, message.id)
      )
    end)

    {
      :ok,
      state
      |> Map.put(:messages, messages)
      |> Map.put(:assocs, assocs)
    }
  end
  
  and_ ~r/^I am logged in as "(?<full_name>[^"]+)"$/, fn state, %{ full_name: full_name } ->
    {name, surname} = String.split(full_name, " ") |> List.to_tuple()
    user = Repo.get_by(User, name: name, surname: surname)
    setup_session(user.email, user.password)
    assert current_path() == "/"

    {
      :ok,
      state
      |> Map.put(:user, user)
    }
  end

  and_ ~r/^I am on the homepage$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  and_ ~r/^I click on View more button of the property "(?<property_title>[^"]+)"$/, fn state, %{ property_title: property_title } ->
    property = Repo.get_by(Property, title: property_title)
    click({:id, "edit-#{property.reference}"})
    {:ok, state}
  end

  and_ ~r/^I click on Send message button$/, fn state ->
    click({:id, "send-message"})
    {:ok, state}
  end

  then_ ~r/^I should be redirected to the message form page$/, fn state ->
    assert current_path() == "/chat/new"
    {:ok, state}
  end

  then_ ~r/^I should see a message form with prefilled title "(?<title>[^"]+)"$/, fn state, %{ title: title } ->
    title_el = find_element(:id, "title")
    assert = visible_text(title_el) == title
    {
      :ok,
      state
      |> Map.put(:title, title)
    }
  end

  then_ ~r/^I should see the new message recipient being "(?<full_name>[^"]+)"$/, fn state, %{ full_name: full_name } ->
    assert visible_in_page? ~r/Message will be sent to #{full_name}/
    {:ok, state}
  end

  when_ ~r/^I fill in the message form with the following information$/, fn state, %{ table_data: table } ->
    content = List.first(table)[:content]
    find_element(:id, "content") |> fill_field(content)
    {
      :ok,
      state
      |> Map.put(:content, content)
    }
  end

  and_ ~r/^I click on Send button$/, fn state ->
    click({:id, "send"})
    {:ok, state}
  end

  then_ ~r/^I should see a success message "(?<message>[^"]+)"$/, fn state, %{ message: message } ->
    assert visible_in_page? ~r/#{message}/
    {:ok, state}
  end

  and_ ~r/^I should be redirected to the messages page$/, fn state ->
    assert current_path() == "/chat"
    {:ok, state}
  end

  and_ ~r/^I should see the message in the messages list$/, fn state ->
    assert visible_in_page? ~r/#{state[:title]}/
    assert visible_in_page? ~r/#{state[:content]}/
    {:ok, state}
  end

  and_ ~r/^I am on the messages page$/, fn state ->
    navigate_to("/chat")
    {:ok, state}
  end

  when_ ~r/^I click on Reply button of an existing message$/, fn state ->
    click({:class, "reply"})
    {:ok, state}
  end

  and_ ~r/^I should see the message content prefilled with "(?<prefill>[^"]+)"$/, fn state, %{ prefill: prefill } ->
    content_el = find_element(:id, "content")
    assert = visible_text(content_el) =~ ~r/#{prefill}/

    {
      :ok,
      state
      |> Map.put(:prefill, prefill)
    }
  end

  then_ ~r/^I should see a list of the messages$/, fn state ->
    assert visible_in_page? ~r/Your messages/
    els = find_all_elements(:class, "message")
    assert length(els) == length(state[:messages])
    state[:messages] |> Enum.each(fn message ->
      assert visible_in_page? ~r/#{message.title}/
    end)
    {:ok, state}
  end

  when_ ~r/^I click on Delete button of an existing message$/, fn state ->
    els = find_all_elements(:class, "delete-button")
    Enum.at(els, 0) |> click()
    {:ok, state}
  end

  then_ ~r/^I should be shown a confirmation modal$/, fn state ->
    :timer.sleep(500)
    assert visible_in_page? ~r/Are you sure you want to delete this message?/
    {:ok, state}
  end

  when_ ~r/^I click Cancel button$/, fn state ->
    els = find_all_elements(:class, "cancel-button")
    Enum.at(els, 0) |> click()
    {:ok, state}
  end

  then_ ~r/^I should remain on the messages page$/, fn state ->
    assert current_path() == "/chat"
    {:ok, state}
  end

  and_ ~r/^I should not see the modal$/, fn state ->
    :timer.sleep(500)
    refute visible_in_page? ~r/Are you sure you want to delete this message?/
    {:ok, state}
  end

  and_ ~r/^I should still see the message$/, fn state ->
    assert visible_in_page? ~r/#{state[:title]}/
    assert visible_in_page? ~r/#{state[:content]}/
    {:ok, state}
  end

  when_ ~r/^I click Confirm button$/, fn state ->
    els = find_all_elements(:class, "confirm-button")
    Enum.at(els, 0) |> click()
    {:ok, state}
  end

  and_ ~r/^I should no longer see the message$/, fn state ->
    messages = state[:messages]

    # Note: We assume here that there exists only one message before
    # running this test so this each-loop is fine.
    messages |> Enum.each(fn message ->
      refute visible_in_page? ~r/#{message.title}/
      refute visible_in_page? ~r/#{message.content}/
    end)

    {:ok, state}
  end

  defp setup_session(email, password) do
    # Note: This is just to bypass the password hashing security
    password = cond do
      password == "" or password == nil -> @password
      true -> password
    end

    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

end
