defmodule NavigationBarContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    random_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:random_email, random_user[:email])
      |> Map.put(:random_password, random_user[:password])
    }
  end

  and_ ~r/^I am logged in as random user$/, fn state ->
    setup_session(state[:random_email], state[:random_password])
    {:ok, state}
  end

  when_ ~r/^I click "(?<link>[^"]+)" in the navigation bar$/, fn state, %{link: link} ->
    find_element(:link_text, link) |> click()
    :timer.sleep(100)
    {:ok, state}
  end

  when_ ~r/^I visit the home page$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  then_ ~r/^I should see "(?<text>[^"]+)" in the navigation bar$/, fn state, %{text: text} ->
    page_source = page_source()
    assert String.contains?(page_source, text)
    {:ok, state}
  end

  then_ ~r/^I should not see "(?<text>[^"]+)" in the navigation bar$/, fn state, %{text: text} ->
    page_source = page_source()
    refute String.contains?(page_source, text)
    {:ok, state}
  end

  then_ ~r/^I should be on the (?<page>[^"]+) page$/, fn state, %{page: page} ->
    expected_path = case page do
      "my properties" -> "/me/properties"
      "favorites" -> "/me/favorites"
      "search" -> "/search"
      _ -> "/"
    end
    assert String.contains?(current_url(), expected_path)
    {:ok, state}
  end

  then_ ~r/^I should see "(?<text>[^"]+)" as the header$/, fn state, %{text: text} ->
    :timer.sleep(100)
    page_source = page_source()
    assert String.contains?(page_source, text)
    {:ok, state}
  end

  then_ ~r/^I should be on the search paged$/, fn state ->
    assert current_path() == "/search"
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
    :timer.sleep(100)
  end
end
