defmodule OwnerProfileContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    owner = Enum.find(table, fn user -> user[:email] == "property.owner@gmail.com" end)
    regular_user = Enum.find(table, fn user -> user[:email] == "regular.user@gmail.com" end)

    {
      :ok,
      state
      |> Map.put(:owner_email, owner[:email])
      |> Map.put(:owner_password, owner[:password])
      |> Map.put(:regular_email, regular_user && regular_user[:email])
      |> Map.put(:regular_password, regular_user && regular_user[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{table_data: table} ->
    owner = Repo.get_by(User, email: state[:owner_email])

    properties =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:properties, properties)
    }
  end

  and_ ~r/^I am logged in as the property owner$/, fn state ->
    setup_session(state[:owner_email], state[:owner_password])
    {:ok, state}
  end

  and_ ~r/^I am logged in as regular user$/, fn state ->
    setup_session(state[:regular_email], state[:regular_password])
    {:ok, state}
  end

  when_ ~r/^I navigate to "(?<property_title>[^"]+)" details page$/,
  fn state, %{property_title: property_title} ->
    property = Enum.find(state.properties, fn p -> p.title == property_title end)
    navigate_to("/properties/#{property.reference}")
    :timer.sleep(500)
    {:ok, state}
  end

  then_ ~r/^I should see "(?<text>[^"]+)" link$/, fn state, %{text: text} ->
    assert visible_in_page?(~r/#{text}/i)
    {:ok, state}
  end

  when_ ~r/^I click "(?<text>[^"]+)" link$/, fn state, %{text: text} ->
    link = find_element(:link_text, text)
    click(link)
    :timer.sleep(500)
    {:ok, state}
  end

  and_ ~r/^I should see "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    :timer.sleep(500) # Small delay to ensure page is loaded

    # First try to find the text in flash messages
    flash_visible = case find_all_elements(:css, ".flash, .alert, [role='alert']") do
      [] -> false
      elements ->
        Enum.any?(elements, fn element ->
          element_text = visible_text(element)
          String.contains?(element_text, argument_one)
        end)
    end

    # If not in flash, check the general page content
    page_contains = String.contains?(page_source(), argument_one)

    assert flash_visible || page_contains,
      "Expected to see '#{argument_one}' on the page or in flash messages but it was not found"

    {:ok, state}
  end

  then_ ~r/^I should be on the owner's profile page$/, fn state ->
    owner = Repo.get_by(User, email: state[:owner_email])
    current_path = current_path()
    expected_path = "/owners/#{owner.id}"
    assert current_path == expected_path,
      "Expected to be on #{expected_path} but was on #{current_path}"
    {:ok, state}
  end

  and_ ~r/^I should see the owner's contact information$/, fn state ->
    owner = Repo.get_by(User, email: state[:owner_email])
    assert visible_in_page?(~r/#{owner.email}/i)
    assert visible_in_page?(~r/#{owner.phone_number}/i)
    {:ok, state}
  end

  and_ ~r/^I click on drop-down menu and update status to unavailable$/, fn state ->
    button = find_element(:css, ".status-dropdown-button")
    click(button)
    :timer.sleep(200)

    option = find_element(:css, "[data-status='unavailable']")
    click(option)
    :timer.sleep(500)

    {:ok, state}
  end

  then_ ~r/^I should be redirected to homepage$/, fn state ->
    current_path = current_path()
    assert current_path == "/",
      "Expected to be redirected to homepage but was on #{current_path}"
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
end
