defmodule PasswordChangeContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  import PropTrackr.Authentication

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table                                                                                
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)                            
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    new_password = "new password cool"

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
      |> Map.put(:new_password, new_password)
    }
  end

  and_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:email], state[:password])
    {:ok, state}
  end

  and_ ~r/^I want to change my password$/, fn state ->
    navigate_to("/me/password")
    {:ok, state}
  end

  and_ ~r/^I enter my current password$/, fn state ->
    fill_field({:id, "current_password"}, state[:password])
    {:ok, state}
  end

  and_ ~r/^I enter my current password invalid$/, fn state ->
    fill_field({:id, "current_password"}, "obviously_invalid")
    {:ok, state}
  end

  and_ ~r/^I enter my new password and confirm it$/, fn state ->
    fill_field({:id, "new_password"}, state[:new_password])
    fill_field({:id, "new_password_confirmation"}, state[:new_password])
    {:ok, state}
  end

  and_ ~r/^I enter my new password and confirm it with a password that does not match$/, fn state ->
    fill_field({:id, "new_password"}, state[:new_password])
    fill_field({:id, "new_password_confirmation"}, "obviously_invalid")
    {:ok, state}
  end

  and_ ~r/^I enter my new password and confirm it that match the current password$/, fn state ->
    fill_field({:id, "new_password"}, state[:password])
    fill_field({:id, "new_password_confirmation"}, state[:password])
    {:ok, state}
  end

  when_ ~r/^I click change password$/, fn state ->
    click({:id, "change_password"})
    {:ok, state}
  end

  then_ ~r/^I should receive a success notification$/, fn state ->
    assert visible_in_page? ~r"Password changed successfully! Please log in again"
    {:ok, state}
  end

  then_ ~r/^I should receive an error message stating invalid current password$/, fn state ->
    assert visible_in_page? ~r"Invalid current password!"
    {:ok, state}
  end

  then_ ~r/^I should receive an error message stating that the new passwords don't match$/, fn state ->
    assert visible_in_page? ~r"New passwords don't match!"
    {:ok, state}
  end

  then_ ~r/^I should receive an error message stating that the new password is the same as the old password$/, fn state ->
    assert visible_in_page? ~r"New password can't be the same as the old password!"
    {:ok, state}
  end

  and_ ~r/^I should be shown the change password form again$/, fn state ->
    assert visible_in_page? ~r"Change your password"
    {:ok, state}
  end

  and_ ~r/^I should be redirected to the login page$/, fn state ->
    assert current_path() == "/login"
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
end
