defmodule ProfileUpdateContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{registered_users: []}
  end

  scenario_finalize fn _status, state ->
    # Enum.each(state[:registered_users], &Repo.delete/1)
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exist following accounts$/, fn state,%{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)
    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  and_ ~r/^I am logged in$/, fn state ->
    navigate_to("/login")

    fill_field({:id, "email"}, state[:email])
    fill_field({:id, "password"}, state[:password])

    click({:id, "login_button"})

    assert visible_in_page? ~r"Successfully logged in!"
    assert current_path() == "/"

    {:ok, state}
  end

  and_ ~r/^I want to update my profile$/, fn state ->
    navigate_to("/me")
    click({:id, "edit_details"})
    assert visible_in_page? ~r"Update your user"
    assert current_path() == "/me/details"

    {:ok, state}
  end

  and_ ~r/^I fill the profile update form with valid details$/, fn state, %{ table_data: table1 }  ->
    table = List.first(table1)
    fill_field({:id, "name"}, table[:name])
    fill_field({:id, "surname"}, table[:surname])
    formatted_date = formatDate(table[:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, table[:phone_number])
    fill_field({:id, "bio"}, table[:bio])

    {:ok, state}
  end

  when_ ~r/^I click update profile$/, fn state ->
    click({:id, "update_profile"})
    {:ok, state}
  end

  then_ ~r/^I should be redirected to profile page$/, fn state ->
    assert current_path() == "/me"
    {:ok, state}
  end

  and_ ~r/^I should see a success notification$/, fn state ->
    assert visible_in_page? ~r"Profile details updated successfully!"
    {:ok, state}
  end

  and_ ~r/^my profile should be updated with new info$/, fn state, %{ table_data: table1 } ->
    table = List.first(table1)

    formatted_date = formatDate(table[:birth_date])

    assert visible_page_text() =~ (table[:name])
    assert visible_page_text() =~(table[:surname])
    assert visible_page_text() =~(table[:birth_date])
    assert visible_page_text() =~(table[:phone_number])
    assert visible_page_text() =~(table[:bio])
    assert visible_page_text() =~(table[:email])

    {:ok, state}
  end

  and_ ~r/^I fill the profile update form with invalid data$/, fn state, %{ table_data: table1 } ->
    table = List.first(table1)
    fill_field({:id, "name"}, table[:name])
    fill_field({:id, "surname"}, table[:surname])
    formatted_date = formatDate(table[:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, table[:phone_number])
    fill_field({:id, "bio"}, table[:bio])
    {:ok, state}
  end

  then_ ~r/^I should see an error message stating invalid phone number$/, fn state ->
    assert visible_page_text() =~ "must be between 3 and 15 digits, without '+' sign"
    {:ok, state}
  end

  then_ ~r/^I should see an error message stating invalid birth date$/, fn state ->
    assert visible_page_text() =~ "Birth date cannot be in the future"

    {:ok, state}
  end

  and_ ~r/^I should still see the profile update form$/, fn state ->
    assert visible_in_page? ~r"Update your user"
    {:ok, state}
  end





  given_ ~r/^I am not logged in$/, fn state ->
    navigate_to("/logout")
    {:ok, state}
  end

  when_ ~r/^I try to access the profile update page$/, fn state ->
    navigate_to("/me/details")
    {:ok, state}
  end

  then_ ~r/^I should be redirected to the login page$/, fn state ->
    assert current_path() == "/login"
    {:ok, state}
  end

  and_ ~r/^I should see an error message stating that I am not logged in$/, fn state ->
    assert visible_in_page? ~r"You are not logged in!"
    {:ok, state}
  end




  defp formatDate(date) do
    [year, month, day] = String.split(date, "-")
    formatted_date = "#{month}-#{day}-#{year}"
    formatted_date
  end
end
