defmodule PropertyDetailsContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    random_user = List.first(table)
    owner = List.last(table)

    {
      :ok,
      state
      |> Map.put(:random_email, random_user[:email])
      |> Map.put(:random_password, random_user[:password])
      |> Map.put(:owner_email, owner[:email])
      |> Map.put(:owner_password, owner[:password])
      |> Map.put(:owner_phone, owner[:phone_number])
    }
  end

  and_ ~r/^I am logged in as random user$/, fn state ->
    setup_session(state[:random_email], state[:random_password])
    {:ok, state}
  end

  and_ ~r/^I am logged in as owner$/, fn state ->
    setup_session(state[:owner_email], state[:owner_password])
    {:ok, state}
  end

  and_ ~r/^the following properties exist$/, fn state, %{table_data: table} ->
    owner = Repo.get_by(User, email: state[:owner_email])

    properties =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:properties, properties)
    }
  end

  when_ ~r/^I visit the homepage$/, fn state ->
    navigate_to("/")
    :timer.sleep(500)
    {:ok, state}
  end

  # and_ ~r/^I click the "View more" button for "(?<title>[^"]+)"$/, fn state, %{title: title} ->
  #   button = find_element(:css, "[data-title='#{title}']")
  #   click(button)
  #   :timer.sleep(500)
  #   {:ok, state}
  # end

  # and_ ~r/^I click the "View more" button for "(?<title>[^"]+)"$/, fn state, %{title: title} ->
  #   # Get the property from state or find it by title
  #   property = state[:property] || Repo.get_by(Property, title: title)

  #   # Find the button using property reference
  #   button = find_element(:id, "view-#{property.reference}")
  #   click(button)
  #   :timer.sleep(500)

  #   {:ok, state}
  # end

  # and_ ~r/^I click the "View more" button for "(?<title>[^"]+)"$/, fn state, %{title: title} ->
  #   property = state[:property] || Repo.get_by(Property, title: title)

  #   # Get current path to determine which selector to use
  #   current_path = current_path()

  #   button = case current_path do
  #     "/me/favorites" ->
  #       find_element(:id, "view-#{property.reference}")
  #     "/me/properties" ->
  #       find_element(:id, "edit-#{property.reference}")
  #     "/" ->  # Home page
  #       find_element(:css, "[data-title='#{title}']")
  #     _ -> # Owner profile or other pages
  #       # Find button within the property card div
  #       find_element(:css, "div:has(h2:contains('#{title}')) .button")
  #   end

  #   click(button)
  #   :timer.sleep(500)

  #   {:ok, state}
  # end

  and_ ~r/^I click the "View more" button for "(?<title>[^"]+)"$/, fn state, %{title: title} ->
    property = state[:property] || Repo.get_by(Property, title: title)

    # Get current path to determine which selector to use
    current_path = current_path()

    button = case current_path do
      "/me/favorites" ->
        find_element(:id, "view-#{property.reference}")
      "/me/properties" ->
        find_element(:id, "edit-#{property.reference}")
      "/owners/" <> _ -> # Owner profile
        # Find the link within the property-card that contains the title
        property_card = find_element(:css, ".property-card")
        find_within_element(property_card, :css, "a[href='/properties/#{property.reference}']")
      "/" ->  # Home page
        find_element(:css, "[data-title='#{title}']")
    end

    click(button)
    :timer.sleep(500)

    {:ok, state}
  end

  when_ ~r/^I visit the property details page directly$/, fn state ->
    property = List.first(state.properties)
    navigate_to("/properties/#{property.reference}")
    :timer.sleep(500)
    {:ok, state}
  end

  then_ ~r/^I should see all property details$/, fn state ->
    property = List.first(state.properties)

    assert visible_in_page?(~r/#{property.title}/)

    assert visible_in_page?(~r/#{property.description}/)

    assert visible_in_page?(~r/#{String.capitalize(to_string(property.type))}/)
    assert visible_in_page?(~r/#{String.capitalize(to_string(property.property_type))}/)
    assert visible_in_page?(~r/#{property.location}/)
    assert visible_in_page?(~r/#{:erlang.float_to_binary(property.price, decimals: 2)}/)
    assert visible_in_page?(~r/#{property.area}/)
    assert visible_in_page?(~r/#{property.room_count}/)
    assert visible_in_page?(~r/#{property.floor}/)
    assert visible_in_page?(~r/#{property.floor_count}/)

    owner = Repo.get_by(User, email: state[:owner_email])
    assert visible_in_page?(~r/#{owner.name}/)
    assert visible_in_page?(~r/#{owner.surname}/)
    assert visible_in_page?(~r/#{owner.email}/)

    {:ok, state}
  end


  and_ ~r/^I should see masked phone number$/, fn state ->
    phone_display = find_element(:id, "phone-number-display")
    masked_number = visible_text(phone_display)
    assert String.length(masked_number) == String.length(state[:owner_phone])
    assert String.replace(masked_number, "x", "") == ""
    {:ok, state}
  end

  when_ ~r/^I click "Show Phone" button$/, fn state ->
    click({:class, "contact-button"})
    :timer.sleep(100)
    {:ok, state}
  end

  then_ ~r/^I should see the owner's actual phone number$/, fn state ->
    phone_display = find_element(:id, "phone-number-display")
    actual_number = visible_text(phone_display)
    assert actual_number == state[:owner_phone]
    {:ok, state}
  end

  and_ ~r/^I should not see the "Show Phone" button anymore$/, fn state ->
    button = find_element(:class, "contact-button")
    refute element_displayed?(button)
    {:ok, state}
  end

  and_ ~r/^I should see the phone number immediately$/, fn state ->
    phone_display = find_element(:id, "phone-number-display")
    actual_number = visible_text(phone_display)
    assert actual_number == state[:owner_phone]
    {:ok, state}
  end

  and_ ~r/^I should not see "Show Phone" button$/, fn state ->
    elements = find_all_elements(:class, "contact-button")
    assert Enum.empty?(elements)
    {:ok, state}
  end

  given_ ~r/^Property Owner has created following property$/, fn state, %{table_data: table} ->
    owner = Repo.get_by(User, email: state[:owner_email])
    data = List.first(table)

    property =
      %Property{}
      |> Property.changeset(Map.put(data, :reference, Ecto.UUID.generate()))
      |> Ecto.Changeset.put_assoc(:user, owner)
      |> Repo.insert!()

    {:ok, Map.put(state, :property, property)}
  end

  then_ ~r/^I should see basic property details$/, fn state ->
    property = state[:property]

    # Basic info that all users should see
    assert visible_in_page?(~r/#{property.title}/)
    assert visible_in_page?(~r/#{property.description}/)
    assert visible_in_page?(~r/#{property.location}/)
    # Price is formatted with 2 decimal places in the template
    assert visible_in_page?(~r/#{:erlang.float_to_binary(property.price/1, decimals: 2)}/)
    assert visible_in_page?(~r/#{property.room_count}/)
    assert visible_in_page?(~r/#{property.area}/)
    # Type and property_type are capitalized in the template
    assert visible_in_page?(~r/#{String.capitalize(to_string(property.type))}/)
    assert visible_in_page?(~r/#{String.capitalize(to_string(property.property_type))}/)

    # Owner's basic info should be visible
    user = Repo.preload(property, :user).user
    assert visible_in_page?(~r/#{user.name}/)
    assert visible_in_page?(~r/#{user.surname}/)
    assert visible_in_page?(~r/#{user.email}/)

    {:ok, state}
  end

  and_ ~r/^I should not see property management controls$/, fn state ->
    # Should not see Edit/Delete buttons
    refute visible_in_page?(~r/Edit Property/)
    refute visible_in_page?(~r/Delete Property/)

    # Should not see status update dropdown
    elements = find_all_elements(:css, ".status-dropdown-button")
    assert Enum.empty?(elements)

    {:ok, state}
  end

  and_ ~r/^I have favorited the "(?<title>[^"]+)"$/, fn state, %{title: title} ->
    user = Repo.get_by(User, email: state[:random_email])
    property = Repo.get_by(Property, title: title)

    %PropTrackr.Favorites.Favorite{}
    |> PropTrackr.Favorites.Favorite.changeset(%{
      user_id: user.id,
      property_id: property.id
    })
    |> Repo.insert!()

    {:ok, state}
  end

  when_ ~r/^I visit my favorites page$/, fn state ->
    navigate_to("/me/favorites")
    :timer.sleep(500)
    {:ok, state}
  end

  and_ ~r/^I have created following property$/, fn state, %{table_data: table} ->
    owner = Repo.get_by(User, email: state[:owner_email])
    data = List.first(table)

    property =
      %Property{}
      |> Property.changeset(Map.put(data, :reference, Ecto.UUID.generate()))
      |> Ecto.Changeset.put_assoc(:user, owner)
      |> Repo.insert!()

    {:ok, Map.put(state, :property, property)}
  end

  when_ ~r/^I visit my properties page$/, fn state ->
    navigate_to("/me/properties")
    :timer.sleep(500)
    {:ok, state}
  end

  and_ ~r/^I should see property management controls$/, fn state ->
    {:ok, state}
  end

  when_ ~r/^I visit Property Owner's profile$/, fn state ->
    owner = Repo.get_by(User, email: state[:owner_email])
    navigate_to("/owners/#{owner.id}")
    :timer.sleep(500)
    {:ok, state}
  end


  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
end
