defmodule SearchOrderContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.NotInterested

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end


  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  given_ ~r/^the following properties exist$/, fn state, %{ table_data: table } ->
    owner_name = "Property"
    owner_surname = "Owner"
    owner = Repo.get_by(User, name: owner_name, surname: owner_surname)

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:email], state[:password])

    user = Repo.get_by(User, email: state.email)
    {
      :ok,
      state
      |> Map.put(:user_id, user.id)
    }
  end

  and_ ~r/^the following not interested settings exist$/, fn state, %{ table_data: table } ->
    Enum.each(table, fn details ->
      title = details[:title]
      property = Repo.get_by(Property, title: title)
      Repo.insert!(%NotInterested{
        user_id: state.user_id,
        property_id: property.id
      })
    end)

    {:ok, state}
  end

  when_ ~r/^I search for properties$/, fn state ->
    navigate_to("/search")
    click({:id, "search_button"})
    {:ok, state}
  end

  then_ ~r/^I should see the following properties in order$/, fn state, %{ table_data: data}  ->
    sorted_titles = Enum.map(data, fn details ->
      details[:title]
    end)

    elements = find_all_elements(:class, "property-card")
    assert length(elements) == 4

    el_ids = Enum.map(elements, fn el ->
      attribute_value(el, "id")
    end)

    properties = Enum.map(el_ids, fn id_str ->
      id = String.split(id_str, "-") |> List.last |> String.to_integer
      Repo.get_by(Property, id: id)
    end)

    # Note: This could be a flaky test if the data is inserted into the database
    # in a favored order. Meaning if all the interested ads are first.
    # Tried switching them around and the test still seems to pass, thus I'll
    # leave it as is.

    property_titles = Enum.map(properties, fn property ->
      property.title
    end)

    assert property_titles == sorted_titles

    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

end
