defmodule SecureAuthContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  # Registration steps reused from UserRegistrationContext
  given_ ~r/^I want to register a new user$/, fn state ->
    navigate_to("/register")
    {:ok, state}
  end

  and_ ~r/^I have such user data$/, fn state, %{table_data: table} ->
    user_data = List.first(table)
    {:ok, state |> Map.put(:user_data, user_data)}
  end

  when_ ~r/^I fill a registration form with valid data$/, fn state ->
    fill_field({:id, "name"}, state[:user_data][:name])
    fill_field({:id, "surname"}, state[:user_data][:surname])
    formatted_date = format_date(state[:user_data][:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, state[:user_data][:phone_number])
    fill_field({:id, "email"}, state[:user_data][:email])
    fill_field({:id, "password"}, state[:user_data][:password])
    fill_field({:id, "confirm_password"}, state[:user_data][:confirm_password])
    {:ok, state}
  end

  and_ ~r/^the password should be stored encrypted in database$/, fn state ->
    user = Repo.get_by(User, email: state[:user_data][:email])
    refute is_nil(user.hashed_password)
    refute user.hashed_password == state[:user_data][:password]
    assert String.starts_with?(user.hashed_password, "$2b$")
    {:ok, state}
  end

  # Account setup steps
  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    first_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, first_user[:email])
      |> Map.put(:password, first_user[:password])
    }
  end

  and_ ~r/^First User has created following property$/, fn state, %{table_data: table} ->
    first_user = Repo.get_by(User, email: state[:email])
    data = List.first(table)

    property =
      %Property{}
      |> Property.changeset(Map.put(data, :reference, Ecto.UUID.generate()))
      |> Ecto.Changeset.put_assoc(:user, first_user)
      |> Repo.insert!()

    {:ok, Map.put(state, :property, property)}
  end

  # Login steps
  when_ ~r/^I am logged in as "(?<email>[^"]+)"$/, fn state, %{email: email} ->
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, "password")
    click({:id, "login_button"})
    {:ok, state}
  end

  and_ ~r/^I try to access edit page for First User's property$/, fn state ->
    property = state[:property]
    navigate_to("/properties/#{property.reference}/edit")
    {:ok, state}
  end

  # # Protected page access steps
  # when_ ~r/^I try to access protected pages:$/, fn state, %{table_data: table} ->
  #   results = Enum.map(table, fn row ->
  #     path = row["path"]
  #     navigate_to(path)
  #     current_path() == "/login" &&
  #       visible_in_page?(~r/You are not logged in!/)
  #   end)

  #   {:ok, Map.put(state, :access_results, results)}
  # end

  when_ ~r/^I try to access properties page$/, fn state ->
    navigate_to("/me/properties")
    {:ok, state}
  end

  when_ ~r/^I try to access favorites page$/, fn state ->
    navigate_to("/me/favorites")
    {:ok, state}
  end

  when_ ~r/^I try to access profile page$/, fn state ->
    navigate_to("/me")
    {:ok, state}
  end

  when_ ~r/^I try to access password page$/, fn state ->
    navigate_to("/me/password")
    {:ok, state}
  end

  when_ ~r/^I try to access details page$/, fn state ->
    navigate_to("/me/details")
    {:ok, state}
  end

  then_ ~r/^I should be redirected to login page$/, fn state ->
    assert current_path() == "/login"
    {:ok, state}
  end

  # Reuse login/logout steps from UserLoginContext and UserLogoutContext
  given_ ~r/^I am logged in$/, fn state ->
    navigate_to("/login")
    fill_field({:id, "email"}, state[:email])
    fill_field({:id, "password"}, state[:password])
    click({:id, "login_button"})
    assert visible_in_page?(~r/Successfully logged in!/)
    assert current_path() == "/"
    {:ok, state}
  end

  when_ ~r/^I click the logout button$/, fn state ->
    refresh_page()
    click({:id, "logout_button"})
    {:ok, state}
  end

  and_ ~r/^I click submit$/, fn state ->
    click({:id, "register_button"})
    :timer.sleep(500)
    {:ok, state}
  end

  then_ ~r/^I should see a success message$/, fn state ->
    assert visible_in_page? ~r/User created successfully./
    {:ok, state}
  end

  and_ ~r/^I should be redirected to the main page$/, fn state ->
    assert current_path() == "/"
    {:ok, state}
  end


  then_ ~r/^I should be redirected to property details page$/, fn state ->
    property = state[:property]
    assert current_path() == "/properties/#{property.reference}"
    {:ok, state}
  end

  and_ ~r/^I should see "(?<argument_one>[^"]+)" message$/,
  fn state, %{argument_one: argument_one} ->
    assert visible_in_page?(~r/#{argument_one}/)
    {:ok, state}
  end

  when_ ~r/^I try to access "(?<path>[^"]+)"$/, fn state, %{path: path} ->
    navigate_to(path)
    {:ok, state}
  end

  then_ ~r/^I should be logged out$/, fn state ->
    assert visible_in_page?(~r/You are successfully logged out/)
    {:ok, state}
  end

  and_ ~r/^I should be redirected to the home page$/, fn state ->
    assert current_path() == "/"
    {:ok, state}
  end

  defp format_date(date) do
    [year, month, day] = String.split(date, "-")
    "#{month}-#{day}-#{year}"
  end
end
