defmodule PropTrackr.Accounts.User do
  use Ecto.Schema
  import Ecto.Changeset

  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User

  schema "users" do
    field :name, :string
    field :surname, :string
    field :birth_date, :date
    field :phone_number, :string
    field :bio, :string
    field :email, :string
    field :password, :string, virtual: true
    field :confirm_password, :string, virtual: true
    field :hashed_password, :string
    field :is_dark_mode, :boolean, default: false

    has_many :properties, PropTrackr.Properties.Property
    has_many :searches, PropTrackr.Search
    has_many :messages, PropTrackr.Messaging.Message

    timestamps()
  end

  def changeset(struct, params \\ %{}) do
    struct
    |> cast(params, [:name, :surname, :birth_date, :phone_number, :bio, :email, :password, :confirm_password, :is_dark_mode])
    |> validate_required([:name, :surname, :birth_date, :phone_number, :email, :password, :confirm_password])
    |> validate_format(:name, ~r/^[a-zA-Z\s-]+$/, message: "must contain only English letters, spaces, or hyphens")
    |> validate_format(:surname, ~r/^[a-zA-Z\s-]+$/, message: "must contain only English letters, spaces, or hyphens")
    |> validate_format(:phone_number, ~r/^\d{3,15}$/, message: "must be between 3 and 15 digits, without '+' sign")
    |> validate_format(:email, ~r/^[^@]+@[^@]+\.[^@]+$/, message: "must contain exactly one '@' sign and be in a valid format")
    |> validate_length(:name, min: 2, max: 50)
    |> validate_length(:surname, min: 2, max: 50)
    |> validate_date(:birth_date)
    |> validate_email_format()
    |> validate_password_confirmation(:password, :confirm_password)
    |> validate_unique_email()
    |> hash_password()
  end

  def update_changeset(struct, params \\ %{}) do
    struct
    |> cast(params, [:name, :surname, :birth_date, :phone_number, :bio])
    |> validate_required([:name, :surname, :birth_date, :phone_number, :bio])
    |> validate_format(:name, ~r/^[a-zA-Z\s-]+$/, message: "must contain only English letters, spaces, or hyphens")
    |> validate_format(:surname, ~r/^[a-zA-Z\s-]+$/, message: "must contain only English letters, spaces, or hyphens")
    |> validate_format(:phone_number, ~r/^\d{3,15}$/, message: "must be between 3 and 15 digits, without '+' sign")
    |> validate_length(:name, min: 2, max: 50)
    |> validate_length(:surname, min: 2, max: 50)
    |> validate_date_update(:birth_date)
  end

  defp validate_date(changeset, date) do
    case get_field(changeset, date) do
      nil -> changeset
      birth_date ->
        cond do
          Date.diff(Date.utc_today(), birth_date) < 0 ->
            add_error(changeset, date, "Birth date cannot be in the future")

          Date.diff(birth_date, ~D[1900-01-01]) < 0 ->
            add_error(changeset, date, "Date is too past")

          true ->
            changeset
        end
    end
    |> validate_unique_email()
  end

  defp validate_date_update(changeset, date) do
    case get_field(changeset, date) do
      nil -> changeset
      birth_date ->
        cond do
          Date.diff(Date.utc_today(), birth_date) < 0 ->
            add_error(changeset, date, "Birth date cannot be in the future")

          Date.diff(birth_date, ~D[1900-01-01]) < 0 ->
            add_error(changeset, date, "Date is too past")

          true ->
            changeset
        end
    end
  end

  def register_changeset(struct, params \\ %{}) do
    changeset(struct, params)
  end

  def login_changeset(struct, params \\ %{}) do
    struct
    |> cast(params, [:email, :password])
    |> validate_required([:email, :password])
  end



  defp validate_password_confirmation(changeset, field_1, field_2) do
    case get_field(changeset, field_1) do
      nil -> changeset
      password ->
        confirm = get_field(changeset, field_2)
        if password == confirm do
          changeset
        else
          add_error(changeset, field_2, "Passwords do not match.")
        end
    end
  end

  defp validate_email_format(changeset) do
    changeset
    |> validate_format(:email, ~r/^[^@\s]+@[^@\s]+\.[^@\s]+$/, message: "must be a valid email address")
    |> validate_length(:email, min: 6, max: 255)
  end

  defp validate_unique_email(changeset) do
    case get_field(changeset, :email) do
      nil -> changeset
      email ->
        case Repo.get_by(User, email: email) do
          nil -> changeset
          _ -> add_error(changeset, :email, "An account with this email already exists.")
        end
    end
  end

  def update_theme_preference(user, is_dark) do
    user
    |> cast(%{is_dark_mode: is_dark}, [:is_dark_mode])
    |> PropTrackr.Repo.update()
  end

  defp hash_password(changeset) do
    case changeset do
      %Ecto.Changeset{valid?: true, changes: %{password: password}} ->
        put_change(changeset, :hashed_password, Bcrypt.hash_pwd_salt(password))
      _ ->
        changeset
    end
  end

end
