defmodule PropTrackrWeb.ChatController do
  use PropTrackrWeb, :controller

  import Ecto.Query, only: [from: 2]
  import Ecto.Changeset
  alias Ecto.Multi

  alias PropTrackr.Repo
  alias PropTrackr.Messaging.{Message, MessageAssociation}
  alias PropTrackr.Properties.Property
  alias PropTrackr.Accounts.User

  def index(conn, _params) do
    current_user = conn.assigns.current_user

    if current_user do
      messages =
        from(
          m in Message,
          join: ma in MessageAssociation,
          on: m.id == ma.message_id,
          join: u in User,
          on: ma.user_id == u.id,
          where: u.id == ^current_user.id,
          order_by: [desc: m.inserted_at],
          preload: [:user],
          select: m
        )
        |> Repo.all()
      render(conn, "index.html", messages: messages)
    else
      conn
      |> put_flash(:error, "You must be logged in to view messages")
      |> redirect(to: "/login")
    end
  end

  def new(conn, %{ "reference" => reference }) do
    current_user = conn.assigns.current_user

    if current_user != nil do
      changeset = Message.changeset(%Message{}, %{})

      advertisement = Repo.get_by(Property, reference: reference)
      changeset = put_change(changeset, :title, "Re: #{advertisement.title}")

      recipient = Repo.get(User, advertisement.user_id)

      render(conn, "new.html", changeset: changeset,
        recipient: recipient, reference: reference,
        title: "Send a new message")
    else
      conn
      |> put_flash(:error, "You must be logged in to send a message")
      |> redirect(to: "/login")
    end
  end

  def new(conn, %{ "reply" => reply }) do
    current_user = conn.assigns.current_user

    if current_user != nil do
      reply_to = Repo.get(Message, reply)

      recipient_id = from(
        ma in MessageAssociation,
        where: ma.message_id == ^reply_to.id,
        where: ma.user_id != ^current_user.id,
        select: ma.user_id
      ) |> Repo.one()

      if current_user.id == recipient_id do
        conn
        |> put_flash(:error, "You cannot reply to your own message")
        |> redirect(to: "/chat")
      else
        recipient = Repo.get(User, recipient_id)

        changeset = Message.changeset(%Message{}, %{})

        new_reply_title = case String.split(reply_to.title, "Re: ") do
          [_, title] -> "Re: #{title}"
          _ -> "Re: #{reply_to.title}"
        end

        new_reply_content = "\n\n---#{reply_to.title}\n\n#{reply_to.content}"

        changeset = put_change(changeset, :title, new_reply_title)
        changeset = put_change(changeset, :content, new_reply_content)

        title = "Reply to '#{reply_to.title}'"

        render(conn, "new.html", changeset: changeset, recipient: recipient, reference: "", title: title)
      end
    else
      conn
      |> put_flash(:error, "You must be logged in to reply to a message")
      |> redirect(to: "/login")
    end
  end

  def new(conn, _params) do
    conn
    |> put_flash(:error, "You must begin a message from an advertisement listing")
    |> redirect(to: "/chat")
  end

  def create(conn, %{ "message" => message }) do
    current_user = conn.assigns.current_user

    if current_user do
      message_assoc =
        Ecto.build_assoc(
          current_user,
          :messages,
          Enum.map(message, fn {k, v} -> {String.to_atom(k), v} end)
        )
      message_changeset = Message.changeset(message_assoc)

      reference = message["reference"]

      recipient = cond do
        reference ->
          from(
            u in User,
            join: p in Property,
            on: u.id == p.user_id,
            where: p.reference == ^reference
          )
          |> Repo.one()
        true -> Repo.get(User, String.to_integer(message["recipient_id"]))
      end

      if not message_changeset.valid? do
        conn
        |> put_flash(:error, 
          cond do
            message_changeset.errors[:title] -> "Invalid title"
            message_changeset.errors[:content] -> "Invalid content"
            true -> "Invalid message"
          end
        )
        |> redirect(to: "/chat")
      else
        case Repo.insert(message_assoc) do
          {:ok, message} ->
            assoc_current_user = MessageAssociation.changeset(%MessageAssociation{})
            assoc_current_user =
              assoc_current_user
              |> put_change(:user_id, current_user.id)
              |> put_change(:message_id, message.id)
              
            assoc_recipient = MessageAssociation.changeset(%MessageAssociation{})
            assoc_recipient =
              assoc_recipient
              |> put_change(:user_id, recipient.id)
              |> put_change(:message_id, message.id)

            Multi.new()
            |> Multi.insert(:assoc_current_user, assoc_current_user)
            |> Multi.insert(:assoc_recipient, assoc_recipient)
            |> Repo.transaction()

            conn
            |> put_flash(:info, "Message sent successfully")
            |> redirect(to: ~p"/chat")
          {:error, changeset} ->
            conn
            |> put_flash(:error, "Error sending message")
            |> render("new.html", changeset: changeset, recipient: recipient, reference: reference)
        end
      end
    else
      conn
      |> put_flash(:error, "You must be logged in to send a message")
      |> redirect(to: "/login")
    end
  end

  def delete(conn, %{ "id" => id }) do
    current_user = conn.assigns.current_user

    if current_user do
      message = Repo.get(Message, id)

      if message.user_id == current_user.id do
        result = Multi.new()
        |> Multi.delete_all(:message_associations, from(ma in MessageAssociation, where: ma.message_id == ^message.id))
        |> Multi.delete(:message, message)
        |> Repo.transaction()

        case result do
          {:ok, _} ->
            conn
            |> put_flash(:info, "Message deleted successfully")
            |> redirect(to: ~p"/chat")
          {:error, _} ->
            conn
            |> put_flash(:error, "Error deleting message")
            |> redirect(to: ~p"/chat")
        end
      else
        conn
        |> put_flash(:error, "You are not allowed to delete this message")
        |> redirect(to: ~p"/chat")
      end
    else
      conn
      |> put_flash(:error, "You must be logged in to delete a message")
      |> redirect(to: ~p"/login")
    end
  end

end
