defmodule PropTrackrWeb.PropertiesController do
use PropTrackrWeb, :controller

import Ecto.Query, only: [from: 2]
alias PropTrackr.Repo
alias PropTrackr.Properties.{Property,  Photo}
alias PropTrackr.Favorites.Favorite
alias PropTrackr.Uploads
alias PropTrackr.NotInterested

def index(conn, _params) do
  current_user = conn.assigns.current_user
  current_user_id = current_user && current_user.id

  properties = from(p in Property,
    left_join: ni in NotInterested,
    on: ni.property_id == p.id and ni.user_id == ^(current_user_id || 0),
    where: p.state == :available,
    order_by: [asc: not is_nil(ni.id), asc: p.inserted_at],
    preload: [:photos],
    select: p)
  |> Repo.all()

  not_interested = if current_user do
    Repo.all(from n in NotInterested,
      where: n.user_id == ^current_user.id,
      select: n.property_id)
    else
      []
  end

  favorites = case conn.assigns.current_user do
    nil -> []
    current_user ->
      Repo.all(
        from f in Favorite,
        where: f.user_id == ^current_user.id,
        select: f.property_id
      )
  end

  render(conn, "index.html",
  properties: properties,
  favorites: favorites,
  not_interested: not_interested,
  is_search: false,
  )
end

def show(conn, %{"reference" => reference}) do
  property = Repo.one(
    from p in Property,
    where: p.reference == ^reference,
    preload: [:user, :photos],
    select: p
  )

  case property do
    nil ->
      conn
      |> put_flash(:error, "Property not found.")
      |> redirect(to: ~p"/")
    property ->
      current_user = conn.assigns.current_user
      can_edit = current_user && current_user.id == property.user_id

      favorites = case current_user do
        nil -> []
        user ->
          Repo.all(
            from f in Favorite,
            where: f.user_id == ^user.id,
            select: f.property_id
          )
      end

      similar_properties = get_similar_properties(property)

      render(conn, "show.html",
        property: property,
        can_edit: can_edit,
        favorites: favorites,
        similar_properties: similar_properties,
      )
  end
end


def new(conn, _params) do
  changeset = Property.changeset(%Property{}, %{})
  render conn, "new.html", changeset: changeset
end


def create(conn, %{"property" => property_params}) do
current_user = conn.assigns.current_user

if current_user == nil do
  conn
  |> put_flash(:error, "You are not logged in!")
  |> redirect(to: "/")
else
  uploaded_files = Map.get(property_params, "photos", [])

  # Add reference to property params
  property_params = property_params |> Map.put("reference", Ecto.UUID.generate())

  # Clean and convert property params
  property_params_with_atoms =
    property_params
    |> Map.delete("photos")
    |> Enum.map(fn {key, value} -> {String.to_atom(key), value} end)
    |> Enum.into(%{})

  # Build property association with user
  property_assoc = Ecto.build_assoc(current_user, :properties, property_params_with_atoms)
  property_changeset = Property.changeset(property_assoc, property_params_with_atoms)

  if length(uploaded_files) < 1 or length(uploaded_files) > 5 do
    conn
    |> put_flash(:error, "Please upload between 1 and 5 photos")
    |> render("new.html", changeset: property_changeset)
  else
    Ecto.Multi.new()
    |> Ecto.Multi.insert(:property, property_changeset)
    |> Ecto.Multi.run(:photos, fn repo, %{property: property} ->

      results =
        uploaded_files
        |> Enum.with_index(1)
        |> Enum.map(fn {upload, index} ->
          case upload do
            %Plug.Upload{} = file ->
              # Save file and create photo record
              filename = Uploads.upload_file(file)

              photo_changeset = Photo.changeset(%Photo{}, %{
                filename: filename,
                order: index,
                property_id: property.id
              })

              case repo.insert(photo_changeset) do
                {:ok, photo} ->
                  {:ok, photo}
                {:error, reason} = error ->
                  error
              end
            _ -> {:error, "Invalid file upload"}
          end
        end)

      case Enum.split_with(results, &(elem(&1, 0) == :ok)) do
        {successful, []} ->
          {:ok, Enum.map(successful, &elem(&1, 1))}
        {_, failed} ->
          {:error, "Failed to create some photos: #{inspect(failed)}"}
      end
    end)
    |> Repo.transaction()
    |> case do
      {:ok, %{property: property, photos: photos}} ->
        conn
        |> put_flash(:info, "Property created successfully!")
        |> redirect(to: ~p"/properties/#{property.reference}")

      {:error, :property, changeset, _} ->
        conn
        |> put_flash(:error, "Failed to create property")
        |> render("new.html", changeset: changeset)

      {:error, :photos, error, _} ->
        conn
        |> put_flash(:error, "Failed to upload photos: #{inspect(error)}")
        |> render("new.html", changeset: property_changeset)
    end
  end
end
end

def edit(conn, %{"reference" => reference}) do
  property = Repo.get_by(Property, reference: reference) |> Repo.preload(:photos)

  case property do
    nil ->
      conn
      |> put_flash(:error, "Property not found.")
      |> redirect(to: ~p"/")
    property ->
      if conn.assigns.current_user && conn.assigns.current_user.id == property.user_id do
        changeset = Property.changeset(property)
        render(conn, "edit.html", property: property, changeset: changeset)
      else
        conn
        |> put_flash(:error, "You don't have permission to edit this property.")
        |> redirect(to: ~p"/properties/#{property.reference}")
      end
  end
end

def update(conn, %{"reference" => reference, "property" => property_params}) do
  property = Repo.get_by(Property, reference: reference) |> Repo.preload(:photos)

  case property do
    nil ->
      conn
      |> put_flash(:error, "Property not found.")
      |> redirect(to: ~p"/")

    property ->
      if conn.assigns.current_user && conn.assigns.current_user.id == property.user_id do
        # Get deletion and upload params upfront
        delete_photos = Map.get(property_params, "photos_to_delete", [])
        new_photos = Map.get(property_params, "photos", [])
        current_photo_count = length(property.photos)

        # Check final photo count before starting transaction
        final_photo_count = current_photo_count - length(delete_photos) + length(new_photos)

        if final_photo_count > 5 do
          conn
          |> put_flash(:error, "Maximum 5 photos allowed")
          |> render("edit.html", property: property, changeset: Property.changeset(property))
        else
          Ecto.Multi.new()
          |> Ecto.Multi.update(:property, Property.changeset(property, property_params))
          |> Ecto.Multi.run(:photos, fn repo, %{property: updated_property} ->
            # Handle photo deletions
            if delete_photos != [] do
              Enum.each(delete_photos, fn photo_id ->
                case Repo.get(Photo, photo_id) do
                  nil -> nil
                  photo ->
                    File.rm(Path.join(Application.app_dir(:proptrackr, "priv/static/uploads"), photo.filename))
                    Repo.delete(photo)
                end
              end)
            end

            # Handle new photo uploads
            results = Enum.with_index(new_photos, current_photo_count - length(delete_photos) + 1)
            |> Enum.map(fn {upload, index} ->
              case upload do
                %Plug.Upload{} = file ->
                  filename = Uploads.upload_file(file)
                  photo_changeset = Photo.changeset(%Photo{}, %{
                    filename: filename,
                    order: index,
                    property_id: updated_property.id
                  })
                  repo.insert(photo_changeset)
                _ -> {:error, "Invalid file upload"}
              end
            end)

            case Enum.find(results, &(elem(&1, 0) == :error)) do
              nil -> {:ok, results}
              {:error, reason} -> {:error, reason}
            end
          end)
          |> Repo.transaction()
          |> case do
            {:ok, %{property: updated_property}} ->
              conn
              |> put_flash(:info, "Property updated successfully.")
              |> redirect(to: ~p"/properties/#{updated_property.reference}")
            {:error, :property, changeset, _} ->
              conn
              |> put_flash(:error, "Failed to update property")
              |> render("edit.html", property: property, changeset: changeset)
            {:error, :photos, error, _} ->
              conn
              |> put_flash(:error, "Failed to update photos: #{inspect(error)}")
              |> render("edit.html", property: property, changeset: Property.changeset(property))
          end
        end
      else
        conn
        |> put_flash(:error, "You don't have permission to update this property.")
        |> redirect(to: ~p"/properties/#{property.reference}")
      end
  end
end

def delete(conn, %{"reference" => reference}) do
  if conn.assigns.current_user == nil do
    conn
    |> put_flash(:error, "You are not logged in!")
    |> redirect(to: "/")
  else
    # property = Repo.get_by(Property, reference: reference)
    property = Repo.get_by(Property, reference: reference) |> Repo.preload(:photos)

    case property do
      nil ->
        conn
        |> put_flash(:error, "Property not found.")
        |> redirect(to: ~p"/")

      property ->
        if conn.assigns.current_user.id == property.user_id do
          Enum.each(property.photos, fn photo ->
            uploads_dir = Application.app_dir(:proptrackr, "priv/static/uploads")
            file_path = Path.join(uploads_dir, photo.filename)
            File.rm(file_path)
          end)

          case Repo.delete(property) do
            {:ok, _deleted_property} ->
              conn
              |> put_flash(:info, "Property deleted successfully.")
              |> redirect(to: ~p"/me/properties")

            {:error, _changeset} ->
              conn
              |> put_flash(:error, "Failed to delete property.")
              |> redirect(to: ~p"/properties/#{property.reference}")
          end
        else
          conn
          |> put_flash(:error, "You don't have permission to delete this property.")
          |> redirect(to: ~p"/properties/#{property.reference}")
        end
    end
  end
end

def update_status(conn, %{"reference" => reference, "state" => state}) do
  case conn.assigns[:current_user] do
    nil ->
      conn
      |> put_status(:unauthorized)
      |> json(%{error: "You must be logged in to perform this action"})

    current_user ->
      property = Repo.get_by(Property, reference: reference)

      case property do
        nil ->
          conn
          |> put_status(:not_found)
          |> json(%{error: "Property not found"})

        property ->
          if current_user.id == property.user_id do
            changeset = Property.state_changeset(property, %{"state" => state})

            case Repo.update(changeset) do
              {:ok, updated_property} ->
                json(conn, %{
                  success: true,
                  message: "Status updated successfully",
                  state: updated_property.state
                })

              {:error, _changeset} ->
                conn
                |> put_status(:unprocessable_entity)
                |> json(%{error: "Failed to update status"})
            end
          else
            conn
            |> put_status(:unauthorized)
            |> json(%{error: "You don't have permission to update this property"})
          end
      end
  end
end

defp get_similar_properties(property) do
  Repo.all(
    from p in Property,
    where: p.state == :available
        and p.id != ^property.id,
    order_by: [
      desc: fragment("abs(? - ?)", p.price, ^property.price),
      desc: fragment("abs(? - ?)", p.room_count, ^property.room_count),
      desc: fragment("abs(? - ?)", p.area, ^property.area),
      desc: fragment("abs(? - ?)", p.floor, ^property.floor)
    ],
    limit: 5,
    select: p
  )
end

defp get_similar_price(%{
  "type" => type,
  "property_type" => property_type,
  "area" => area,
  "room_count" => room_count,
  "floor" => floor,
}) do
  area = if is_binary(area), do: String.to_float(area), else: area
  room_count = if is_binary(room_count), do: String.to_integer(room_count), else: room_count
  floor = if is_binary(floor), do: String.to_integer(floor), else: floor
  type = if is_binary(type), do: String.to_existing_atom(type), else: type
  property_type = if is_binary(property_type), do: String.to_existing_atom(property_type), else: property_type

  query =
    from p in Property,
      where: p.state == :available and
              p.type == ^type and
              p.property_type == ^property_type and
              fragment("ABS(? - ?) <= ?", p.area, ^area, 10) and
              fragment("ABS(? - ?) <= ?", p.room_count, ^room_count, 1) and
              fragment("ABS(? - ?) <= ?", p.floor, ^floor, 1),
      order_by: [
        desc: fragment("ABS(? - ?)", p.area, ^area),
        desc: fragment("ABS(? - ?)", p.room_count, ^room_count),
        desc: fragment("ABS(? - ?)", p.floor, ^floor)
      ],
      limit: 5,
      select: p.price

  similar_prices = Repo.all(query) |> Enum.reject(&is_nil/1)

  if similar_prices == [] do
    nil
  else
    Enum.sum(similar_prices) / length(similar_prices) |> Float.round(0)
  end
end



def calculate_price(conn, %{
  "type" => type,
  "property_type" => property_type,
  "area" => area,
  "room_count" => room_count,
  "floor" => floor,
  "floor_count" => floor_count}) do
  try do
    type = if is_binary(type), do: String.to_existing_atom(type), else: type
    property_type = if is_binary(property_type), do: String.to_existing_atom(property_type), else: property_type
    area = if is_binary(area), do: String.to_float(area), else: area
    room_count = if is_binary(room_count), do: String.to_integer(room_count), else: room_count
    floor = if is_binary(floor), do: String.to_integer(floor), else: floor
    floor_count = if is_binary(floor_count), do: String.to_integer(floor_count), else: floor_count

    # Use helper function to find average price of similar properties
    similar_price = get_similar_price(%{
      "type" => type,
      "property_type" => property_type,
      "area" => area,
      "room_count" => room_count,
      "floor" => floor
    })

    calculated_price =
      case similar_price do
        nil ->
          # Fallback to formula-based calculation if no similar properties found
          sqr_price =
            case {type, property_type} do
              {:rent, :apartment} -> 10
              {:rent, :house} -> 8
              {:rent, :other} -> 6
              {:sell, :apartment} -> 15
              {:sell, :house} -> 12
              {:sell, :other} -> 9
              _ -> 8
            end

          area_base = sqr_price * area
          bonus_room = 0.7 + (room_count * 0.1)
          bonus_floor = 0.5 + (floor / (floor_count + 1.5))
          Float.round(area_base * bonus_room * bonus_floor, 0)

        _ ->
          # Use the average price from similar properties
          similar_price
      end

    json(conn, %{calculated_price: calculated_price})
  rescue
    e in ArgumentError ->
      conn
      |> put_status(:unprocessable_entity)
      |> json(%{error: "Invalid input data. Please ensure all fields are correctly filled with proper types."})
    _ ->
      conn
      |> put_status(:unprocessable_entity)
      |> json(%{error: "An unexpected error occurred during calculation."})
  end
end



def calculate_price(conn, _params) do
  conn
  |> put_status(:unprocessable_entity)
  |> json(%{error: "Invalid input data. Please ensure all fields are correctly filled with proper types."})
end
end
