defmodule PropTrackrWeb.SearchController do
  use PropTrackrWeb, :controller

  import Ecto.Query, only: [from: 2]

  alias PropTrackr.Repo
  alias PropTrackr.Search
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite
  alias PropTrackr.NotInterested

  def index(conn, _params) do
    current_user = conn.assigns.current_user

    if current_user do
      min_price = from(p in Property, select: min(p.price)) |> Repo.one
      max_price = from(p in Property, select: max(p.price)) |> Repo.one

      min_rooms = 0
      max_rooms = from(p in Property, select: max(p.room_count)) |> Repo.one

      locations = from(p in Property, select: p.location) |> Repo.all
      locations = locations |> Enum.filter(fn loc -> length(String.split(loc, ",")) >= 2 end)
      cities = Enum.map(locations, fn location -> String.split(location, ",") |> hd |> String.trim() end)
      cities = Enum.uniq(cities)
      countries = Enum.map(locations, fn location -> String.split(location, ",") |> tl |> hd |> String.trim() end)
      countries = Enum.uniq(countries)

      changeset = Search.changeset(%Search{})

      current_user = conn.assigns.current_user
      previous_searches = case current_user do
        nil -> []
        _ ->
          Repo.all(
            from s in Search,
            where: s.user_id == ^current_user.id,
            order_by: [desc: s.inserted_at]
          )
      end

      render(
        conn, "search.html",
        changeset: changeset,
        min_price: min_price,
        max_price: max_price,
        min_rooms: min_rooms,
        max_rooms: max_rooms,
        cities: cities,
        countries: countries,
        previous_searches: previous_searches
      )
    else
      conn
      |> put_flash(:error, "You are not logged in")
      |> redirect(to: "/login")
    end
  end

  defp save_search(conn, changeset, current_user) do
    saved = if current_user do
      searches = Repo.all(
        from s in Search,
        where: s.user_id == ^current_user.id,
        order_by: [asc: s.inserted_at]
      )

      if length(searches) >= 5 do
        case Repo.delete(Enum.at(searches, 0)) do
          {:ok, _search} -> :ok
          {:error, _changeset} -> :error
        end
      end

      search_assoc = Ecto.build_assoc(current_user, :searches, changeset.changes)
      search_changeset = Search.changeset(search_assoc, changeset.changes)

      case Repo.insert(search_changeset) do
        {:ok, _search} -> :ok
        {:error, _changeset} -> :error
      end
    else
      :ok
    end

    case saved do
      :ok -> conn
      :error -> conn
        |> put_flash(:error, "Unknown error saving search")
    end
  end

  defp render_search(conn, properties, header_text, favorites) do
    render(
      conn, "index.html",
      properties: properties,
      header_text: header_text,
      is_search: true,
      favorites: favorites
    )
  end

  def create(conn, %{ "search" => search }) do
    current_user = conn.assigns.current_user

    if current_user do
      changeset = Search.changeset(%Search{}, search)
      query = Search.create_query(changeset.changes, current_user)

      conn = conn |> save_search(changeset, current_user)

      properties = query |> Repo.all()

      favorites = case conn.assigns.current_user do
        nil -> []
        current_user ->
          Repo.all(
            from f in Favorite,
            where: f.user_id == ^current_user.id,
            select: f.property_id
          )
      end

      render_search(conn, properties, "Search results", favorites)
    else
      conn
      |> put_flash(:error, "You are not logged in")
      |> redirect(to: "/login")
    end
  end

  def create(conn, %{ "simple" => _simple, "search_term" => search_term }) do
    current_user = conn.assigns.current_user
    current_user_id = current_user && current_user.id

    cond do
      search_term == "" ->
        conn
        |> put_flash(:error, "Please insert a search term")
        |> redirect(to: ~p"/")

      String.length(search_term) < 3 ->
        conn
        |> put_flash(:error, "Search term should be at least 3 characters")
        |> redirect(to: ~p"/")

      true -> 
        properties = from(
          p in Property,
          left_join: ni in NotInterested,
          on: ni.property_id == p.id and ni.user_id == ^(current_user_id || 0),
          order_by: [asc: not is_nil(ni.id), asc: p.state],
          where: p.state != :unavailable,
          where: ilike(p.location, ^"%#{search_term}%") or
                 ilike(p.description, ^"%#{search_term}%") or
                 ilike(p.title, ^"%#{search_term}%"),
          preload: [:photos],
          select: p,
        ) |> Repo.all()

        favorites = case conn.assigns.current_user do
          nil -> []
          current_user ->
            Repo.all(
              from f in Favorite,
              where: f.user_id == ^current_user.id,
              select: f.property_id
            )
        end

        header_text = "Search results for '#{search_term}'"

        render_search(conn, properties, header_text, favorites)
    end
  end

  def show(conn, %{ "id" => id }) do
    current_user = conn.assigns.current_user

    if current_user do
      search = from(
          s in Search,
          where: s.id == ^id,
          where: s.user_id == ^current_user.id)
      |> Repo.one()

      if search == nil do
        conn
        |> put_flash(:error, "Search not found")
        |> redirect(to: ~p"/search")
      else
        search = Search.changeset(search)
        query = Search.create_query(search.data, current_user)
        properties = query |> Repo.all()

        favorites =  Repo.all(
          from f in Favorite,
          where: f.user_id == ^current_user.id,
          select: f.property_id
        )

        render(
          conn, "index.html",
          properties: properties,
          header_text: "Search results",
          is_search: true,
          favorites: favorites
        )
      end
    else
      conn
      |> put_flash(:error, "You are not logged in")
      |> redirect(to: "/login")
    end
  end
end
