defmodule PropTrackrWeb.ChatControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.{Property, Photo}
  alias PropTrackr.Messaging.{Message, MessageAssociation}
  import Ecto.Query, only: [from: 2]
  alias PropTrackr.Repo

  @password "testing"

  setup do
    user = %User{
      name: "Test",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    user = Repo.insert!(user)

    other_user = %User{
      name: "Other",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "111",
      bio: "Hey",
      email: "other.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    other_user = Repo.insert!(other_user)

    unrelevant_user = %User{
      name: "Unrelevant",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "222",
      bio: "Hi",
      email: "unrelevant@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    unrelevant_user = Repo.insert!(unrelevant_user)

    property = %Property{
      reference: Ecto.UUID.generate(),
      title: "Test Property",
      description: "Test Description",
      type: :sell,
      property_type: :house,
      state: :available,
      location: "Test Location",
      room_count: 3,
      area: 100.0,
      floor: 2,
      floor_count: 5,
      price: 500000.0,
      user_id: other_user.id
    }
    property = Repo.insert!(property)

    {:ok, %{ user: user, other_user: other_user, unrelevant_user: unrelevant_user, property: property }}
  end

  test "Authenticated user should be able to send a message (also tests FR-24)", %{ conn: conn, user: user, other_user: other_user, property: property } do
    conn = conn |> setup_session(user)
    conn = post(conn, "/chat", %{"message": %{
      "title": "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      "reference": property.reference
    }})

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Message sent successfully"

    messages = Repo.all(Message)
    assert length(messages) == 1
    assert Enum.at(messages, 0).title == "Test Message title"
    assert Enum.at(messages, 0).content == "Test Content that is long enought for the changeset"
    assert Enum.at(messages, 0).user_id == user.id

    assert html_response(conn, 200) =~ "Test Message title"
    assert html_response(conn, 200) =~ "Test Content that is long enought for the changeset"

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 2
    assert Enum.at(assocs, 0).user_id == user.id
    assert Enum.at(assocs, 0).message_id == Enum.at(messages, 0).id
    assert Enum.at(assocs, 1).user_id == property.user_id
    assert Enum.at(assocs, 1).message_id == Enum.at(messages, 0).id
  end

  test "Authenticated user should be able to reply to a message (also tests FR-24)", %{ conn: conn, user: user, other_user: other_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    # Note: Initial message is from user, replying to them as other_user

    conn = conn |> setup_session(other_user)
    conn = post(conn, "/chat", %{"message": %{
      "title": "Reply Message title",
      "content": "Test Reply Content that is long enought for the changeset",
      "recipient_id": "#{user.id}",
    }})

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Message sent successfully"

    messages = Repo.all(Message)
    assert length(messages) == 2
    assert Enum.at(messages, 1).title == "Reply Message title"
    assert Enum.at(messages, 1).content == "Test Reply Content that is long enought for the changeset"
    assert Enum.at(messages, 1).user_id == other_user.id

    # Note: Initial message should also be displayed
    assert html_response(conn, 200) =~ "Test Message title"
    assert html_response(conn, 200) =~ "Test Content that is long enought for the changeset"
    assert html_response(conn, 200) =~ "Reply Message title"
    assert html_response(conn, 200) =~ "Test Reply Content that is long enought for the changeset"

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 4

    assert Enum.at(assocs, 2).user_id == other_user.id
    assert Enum.at(assocs, 2).message_id == Enum.at(messages, 1).id
    assert Enum.at(assocs, 3).user_id == user.id
    assert Enum.at(assocs, 3).message_id == Enum.at(messages, 1).id
  end

  test "Unauthenticated user should not be able to send a message", %{ conn: conn, user: user, other_user: other_user, property: property } do
    conn = post(conn, "/chat", %{"message": %{
      "title": "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      "reference": property.reference
    }})

    assert redirected_to(conn) == "/login"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "You must be logged in to send a message"

    messages = Repo.all(Message)
    assert length(messages) == 0

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 0
  end

  test "Unauthenticated user should not be able to reply to a message", %{ conn: conn, user: user, other_user: other_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    conn = post(conn, "/chat", %{"message": %{
      "title": "Reply Message title",
      "content": "Test Reply Content that is long enought for the changeset",
      "recipient_id": "#{user.id}",
    }})

    assert redirected_to(conn) == "/login"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "You must be logged in to send a message"

    messages = Repo.all(Message)
    assert length(messages) == 1

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 2
  end

  test "Message fields should be validated properly", %{ conn: conn, user: user, other_user: other_user, property: property } do
    conn = conn |> setup_session(user)
    conn = post(conn, "/chat", %{"message": %{
      "title": "",
      "content": "Test Content that is long enought for the changeset",
      "reference": property.reference
    }})

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Invalid title"

    messages = Repo.all(Message)
    assert length(messages) == 0

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 0

    conn = post(conn, "/chat", %{"message": %{
      "title": "Test Message title",
      "content": "",
      "reference": property.reference
    }})

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Invalid content"

    messages = Repo.all(Message)
    assert length(messages) == 0

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 0
  end

  test "Unrelevant user should not be able to see other users' messages", %{ conn: conn, user: user, other_user: other_user, unrelevant_user: unrelevant_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    conn = conn |> setup_session(unrelevant_user)
    conn = get(conn, "/chat")

    refute html_response(conn, 200) =~ "Test Message title"
  end

  test "Authenticated user should see no messages if they have none", %{ conn: conn, user: user, other_user: other_user, property: property } do
    conn = conn |> setup_session(user)
    conn = get(conn, "/chat")

    assert html_response(conn, 200) =~ "No messages"

    messages = Repo.all(Message)
    assert length(messages) == 0
  end

  test "Authenticated user should be able to delete their own messages", %{ conn: conn, user: user, other_user: other_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    conn = conn |> setup_session(user)
    conn = get(conn, "/chat")

    conn = get(conn, "/chat/delete/#{message.id}")

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Message deleted successfully"

    messages = Repo.all(Message)
    assert length(messages) == 0

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 0
  end

  test "Unauthenticated user should not be able to delete a message", %{ conn: conn, user: user, other_user: other_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    conn = get(conn, "/chat/delete/#{message.id}")

    assert redirected_to(conn) == "/login"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "You must be logged in to delete a message"

    messages = Repo.all(Message)
    assert length(messages) == 1

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 2
  end

  test "Authenticated user should not be able to delete a message that has been sent to them", %{ conn: conn, user: user, other_user: other_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: other_user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: other_user.id,
      message_id: message.id,
    })

    conn = conn |> setup_session(user)
    conn = get(conn, "/chat")

    conn = get(conn, "/chat/delete/#{message.id}")

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "You are not allowed to delete this message"

    messages = Repo.all(Message)
    assert length(messages) == 1

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 2
  end

  test "Unrelateed user should not be able to delete any message", %{ conn: conn, user: user, other_user: other_user, unrelevant_user: unrelevant_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    conn = conn |> setup_session(unrelevant_user)
    conn = get(conn, "/chat")

    conn = get(conn, "/chat/delete/#{message.id}")

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "You are not allowed to delete this message"

    messages = Repo.all(Message)
    assert length(messages) == 1

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 2
  end
  
  test "When authenticated user deletes a message, it should not delete any other messages or associations", %{ conn: conn, user: user, other_user: other_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    other_message = Repo.insert!(%Message{
      title: "Other Message title",
      "content": "Other Content that is long enought for the changeset",
      user_id: other_user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: other_user.id,
      message_id: other_message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: other_message.id,
    })

    conn = conn |> setup_session(user)
    conn = get(conn, "/chat")

    conn = get(conn, "/chat/delete/#{message.id}")

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Message deleted successfully"

    messages = Repo.all(Message)
    assert length(messages) == 1
    assert Enum.at(messages, 0).title == "Other Message title"
    assert Enum.at(messages, 0).content == "Other Content that is long enought for the changeset"

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 2
  end

  test "Authenticated user should not be able to delete someone else's message", %{ conn: conn, user: user, other_user: other_user, property: property } do
    message = Repo.insert!(%Message{
      title: "Test Message title",
      "content": "Test Content that is long enought for the changeset",
      user_id: other_user.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: other_user.id,
      message_id: message.id,
    })
    Repo.insert!(%MessageAssociation{
      user_id: property.user_id,
      message_id: message.id,
    })

    conn = conn |> setup_session(user)
    conn = get(conn, "/chat")

    conn = get(conn, "/chat/delete/#{message.id}")

    assert redirected_to(conn) == "/chat"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "You are not allowed to delete this message"

    messages = Repo.all(Message)
    assert length(messages) == 1

    assocs = Repo.all(MessageAssociation)
    assert length(assocs) == 2
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end

end
