defmodule PropTrackrWeb.MyFavoritesControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite
  alias PropTrackr.Repo
  import Ecto.Query, only: [from: 2]

  @password "password123"

  setup do
    # Create owner
    owner = %User{
      name: "Property",
      surname: "Owner",
      birth_date: ~D[2000-01-01],
      phone_number: "111",
      bio: "Hey",
      email: "property.owner@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    owner = Repo.insert!(owner)

    # Create random user
    random_user = %User{
      name: "Random",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Hi",
      email: "random.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    random_user = Repo.insert!(random_user)

    # Create test properties
    property1 = %Property{
      reference: Ecto.UUID.generate(),
      title: "Really cool property",
      description: "Selling this really really house",
      type: :sell,
      property_type: :house,
      state: :available,
      location: "London",
      room_count: 3,
      area: 100.0,
      floor: 2,
      floor_count: 5,
      price: 500000.0,
      user_id: owner.id
    }
    property1 = Repo.insert!(property1)

    property2 = %Property{
      reference: Ecto.UUID.generate(),
      title: "Another property",
      description: "Another cool house",
      type: :sell,
      property_type: :house,
      state: :available,
      location: "Paris",
      room_count: 4,
      area: 120.0,
      floor: 1,
      floor_count: 4,
      price: 600000.0,
      user_id: owner.id
    }
    property2 = Repo.insert!(property2)

    {:ok, %{
      owner: owner,
      random_user: random_user,
      property1: property1,
      property2: property2
    }}
  end

  test "displays user's favorite properties when authenticated", %{
    conn: conn,
    random_user: user,
    property1: property1
  } do

    Repo.insert!(%Favorite{user_id: user.id, property_id: property1.id})

    db_favorite = Repo.one(from f in Favorite,
    where: f.user_id == ^user.id and f.property_id == ^property1.id,
    preload: [:property])
    assert db_favorite.property.title == property1.title

    conn = conn |> setup_session(user) |> get(~p"/me/favorites")

    response = html_response(conn, 200)
    assert response =~ "My Favorite Properties"
    assert response =~ property1.title
    assert response =~ property1.description
    assert response =~ property1.location
    assert response =~ Integer.to_string(property1.room_count)
  end

  test "shows empty message when user has no favorites", %{
    conn: conn,
    random_user: user
  } do

    favorites_count = Repo.one(from f in Favorite,
      where: f.user_id == ^user.id,
      select: count(f.id))
    assert favorites_count == 0

    conn = conn |> setup_session(user) |> get(~p"/me/favorites")

    response = html_response(conn, 200)
    assert response =~ "My Favorite Properties"
    assert response =~ "You haven't favorited any properties yet"

    #double check
    assert Repo.one(from f in Favorite,
      where: f.user_id == ^user.id,
      select: count(f.id)) == 0
  end

  test "only shows favorites for current user", %{
    conn: conn,
    random_user: user1,
    owner: user2,
    property1: property1,
    property2: property2
  } do

    favorite1 = Repo.insert!(%Favorite{user_id: user1.id, property_id: property1.id})
    favorite2 = Repo.insert!(%Favorite{user_id: user2.id, property_id: property2.id})

    assert Repo.get(Favorite, favorite1.id) != nil
    assert Repo.get(Favorite, favorite2.id) != nil

    user1_favorites = Repo.all(from f in Favorite,
    where: f.user_id == ^user1.id,
    preload: [:property])
    assert length(user1_favorites) == 1
    assert hd(user1_favorites).property.title == property1.title

    conn = conn |> setup_session(user1) |> get(~p"/me/favorites")

    response = html_response(conn, 200)
    assert response =~ property1.title
    refute response =~ property2.title

    assert Repo.one(from f in Favorite, select: count(f.id)) == 2
    final_user1_favorites = Repo.all(from f in Favorite,
      where: f.user_id == ^user1.id,
      preload: [:property])
    assert length(final_user1_favorites) == 1
    assert hd(final_user1_favorites).property.title == property1.title
  end

  test "redirects to login when user is not authenticated", %{conn: conn} do
    initial_favorites_count = Repo.one(from f in Favorite, select: count(f.id))
    conn = get(conn, ~p"/me/favorites")

    assert redirected_to(conn) == ~p"/login"
    assert get_flash(conn, :error) == "You are not logged in!"

    assert Repo.one(from f in Favorite, select: count(f.id)) == initial_favorites_count
  end

  test "hides properties that become unavailable from favorites view", %{
    conn: conn,
    random_user: user,
    property1: property
  } do
    Repo.insert!(%Favorite{user_id: user.id, property_id: property.id})

    conn = conn |> setup_session(user) |> get(~p"/me/favorites")
    response = html_response(conn, 200)
    assert response =~ property.title

    property
    |> Property.state_changeset(%{state: :unavailable})
    |> Repo.update!()

    updated_property = Repo.get(Property, property.id)
    assert updated_property.state == :unavailable

    conn = get(conn, ~p"/me/favorites")
    response = html_response(conn, 200)
    refute response =~ property.title
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
