defmodule PropTrackrWeb.NavigationTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Repo

  @password "1234"

  setup do
    user = %User{
      name: "Test",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Test bio",
      email: "test.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    user = Repo.insert!(user)
    {:ok, %{user: user}}
  end

  describe "navigation bar" do
    test "authenticated user sees all navigation links", %{conn: conn, user: user} do
      conn = conn |> setup_session(user) |> get("/")
      html_response = html_response(conn, 200)

      # Check presence of all required navigation items
      assert html_response =~ "My Properties"
      assert html_response =~ "My Favorites"
      assert html_response =~ "Search"
      assert html_response =~ "Profile"
      assert html_response =~ "Logout"
      refute html_response =~ "Log in"
    end

    test "unauthenticated user sees limited navigation", %{conn: conn} do
      conn = get(conn, "/")
      html_response = html_response(conn, 200)

      refute html_response =~ "My Properties"
      refute html_response =~ "My Favorites"
      refute html_response =~ "Profile"
      refute html_response =~ "Logout"
      assert html_response =~ "Log in"
    end

    test "clicking My Properties navigates to properties page", %{conn: conn, user: user} do
      conn = conn |> setup_session(user) |> get("/me/properties")
      assert html_response(conn, 200) =~ "My Properties"
    end

    test "clicking My Favorites navigates to favorites page", %{conn: conn, user: user} do
      conn = conn |> setup_session(user) |> get("/me/favorites")
      assert html_response(conn, 200) =~ "My Favorite Properties"
    end

    test "clicking Search navigates to search page", %{conn: conn, user: user} do
      conn = conn |> setup_session(user) |> get("/search")
      assert html_response(conn, 200) =~ "Search for properties"
    end

    test "unauthenticated user is redirected from protected pages", %{conn: conn} do
      # Try accessing My Properties
      conn = get(conn, "/me/properties")
      assert redirected_to(conn) == "/login"
      assert get_flash(conn, :error) == "You are not logged in!"

      # Try accessing My Favorites
      conn = get(build_conn(), "/me/favorites")
      assert redirected_to(conn) == "/login"
      assert get_flash(conn, :error) == "You are not logged in!"
    end
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get(conn, redirected_to(conn))
    conn
  end
end
