defmodule PropTrackrWeb.OwnerProfileControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite
  alias PropTrackr.Repo
  import Ecto.Query, only: [from: 2]

  @password "password123"

  setup do
    # Create owner
    owner = %User{
      name: "Property",
      surname: "Owner",
      birth_date: ~D[2000-01-01],
      phone_number: "111",
      bio: "Hey",
      email: "property.owner@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    owner = Repo.insert!(owner)

    # Create regular user
    regular_user = %User{
      name: "Regular",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "222",
      bio: "Hello",
      email: "regular.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    regular_user = Repo.insert!(regular_user)

    # Create test property
    property = %Property{
      reference: Ecto.UUID.generate(),
      title: "Cool Property",
      description: "Amazing house",
      type: :sell,
      property_type: :house,
      state: :available,
      location: "London",
      room_count: 3,
      area: 100.0,
      floor: 2,
      floor_count: 5,
      price: 500000.0,
      user_id: owner.id
    }
    property = Repo.insert!(property)

    {:ok, %{
      owner: owner,
      regular_user: regular_user,
      property: property
    }}
  end

  test "shows owner profile with properties when owner has available properties", %{
    conn: conn,
    owner: owner,
    property: property
  } do
    conn = get(conn, ~p"/owners/#{owner.id}")

    response = html_response(conn, 200)
    assert response =~ owner.name
    assert response =~ owner.email
    assert response =~ owner.phone_number
    assert response =~ owner.bio
    assert response =~ property.title
    assert response =~ property.description
    assert response =~ property.location
    assert response =~ Integer.to_string(property.room_count)
  end

  test "shows owner profile with properties when logged in as different user", %{
    conn: conn,
    owner: owner,
    regular_user: user,
    property: property
  } do
    conn = conn |> setup_session(user) |> get(~p"/owners/#{owner.id}")

    response = html_response(conn, 200)
    assert response =~ owner.name
    assert response =~ property.title
  end

  test "shows owner profile with properties when not logged in", %{
    conn: conn,
    owner: owner,
    property: property
  } do
    conn = get(conn, ~p"/owners/#{owner.id}")

    response = html_response(conn, 200)
    assert response =~ owner.name
    assert response =~ property.title
  end

  test "redirects when owner has no available properties", %{
    conn: conn,
    owner: owner,
    property: property
  } do
    # Make property unavailable
    property
    |> Property.state_changeset(%{state: :unavailable})
    |> Repo.update!()

    conn = get(conn, ~p"/owners/#{owner.id}")

    assert redirected_to(conn) == ~p"/"
    assert get_flash(conn, :error) == "Profile not accessible or does not exist."
  end

  test "redirects when owner id does not exist", %{conn: conn} do
    conn = get(conn, ~p"/owners/999999")

    assert redirected_to(conn) == ~p"/"
    assert get_flash(conn, :error) == "Profile not accessible or does not exist."
  end

  test "shows favorite status for logged in user", %{
    conn: conn,
    owner: owner,
    regular_user: user,
    property: property
  } do
    # Create favorite
    Repo.insert!(%Favorite{
      user_id: user.id,
      property_id: property.id
    })

    conn = conn |> setup_session(user) |> get(~p"/owners/#{owner.id}")

    response = html_response(conn, 200)
    assert response =~ "★" # Favorited symbol
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
