defmodule PropTrackrWeb.ProfileUpdateControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Repo

  @valid_attrs %{
    name: "John",
    surname: "Doe",
    birth_date: ~D[1990-01-01],
    phone_number: "123456789",
    email: "john.doe@example.com",
    password: "1234",
    confirm_password: "1234",
    hashed_password: Bcrypt.hash_pwd_salt("1234"),
    bio: "Software Developer"
  }

  @normal_attrs %{
    name: "Test",
    surname: "Test",
    birth_date: ~D[2001-06-06],
    phone_number: "5555555",
    bio: "Testestest34273094817203948"
  }

  @invalid_attrs %{
    name: "1234",
    surname: "",
    birth_date: ~D[2500-01-01],
    phone_number: "invalid",
    bio: "Test"
  }


  setup do
    user = %User{
      name: "John",
      surname: "Doe",
      birth_date: ~D[1990-01-01],
      phone_number: "123456789",
      email: "john.doe@example.com",
      hashed_password: Bcrypt.hash_pwd_salt("1234"),
      bio: "Software Developer"
    }
    user = Repo.insert!(user)

    {:ok, %{user: user}}
  end

  test "1.6 Authenticated user should be able to update their details", %{conn: conn, user: user} do
    conn = setup_session(conn, user)

    conn = conn
      |> post("/me/details", %{
        user: %{
          name: @normal_attrs[:name],
          surname: @normal_attrs[:surname],
          birth_date: @normal_attrs[:birth_date],
          phone_number: @normal_attrs[:phone_number],
          bio: @normal_attrs[:bio],
        }
      })

    assert redirected_to(conn) == "/me"
    conn = get(conn, redirected_to(conn))
    assert html_response(conn, 200)
    assert get_flash(conn, :info) =~ ~r/Profile details updated successfully/

    updated_user = Repo.get(User, user.id)
    assert updated_user.name == @normal_attrs[:name]
    assert updated_user.surname == @normal_attrs[:surname]
    assert updated_user.birth_date == @normal_attrs[:birth_date]
    assert updated_user.phone_number == @normal_attrs[:phone_number]
    assert updated_user.bio == @normal_attrs[:bio]
  end

  test "1.6 Authenticated user should not be able to update data for invalid name", %{conn: conn, user: user} do
    conn = setup_session(conn, user)

    conn = conn
      |> post("/me/details", %{
        user: %{
          name: @invalid_attrs[:name],
        }
      })

    assert html_response(conn, 200) =~ "Update your user"

    unchanged_user = Repo.get(User, user.id)
    refute unchanged_user.name == @invalid_attrs[:name]
  end

  test "1.6 Authenticated user should not be able to update data for invalid surname", %{conn: conn, user: user} do
    conn = setup_session(conn, user)

    conn = conn
      |> post("/me/details", %{
        user: %{
          surname: @invalid_attrs[:surname],
        }
      })

    assert html_response(conn, 200) =~ "Update your user"

    unchanged_user = Repo.get(User, user.id)
    refute unchanged_user.surname == @invalid_attrs[:surname]
  end

  test "1.6 Authenticated user should not be able to update data for a future birth date", %{conn: conn, user: user} do
    conn = setup_session(conn, user)

    conn = conn
      |> post("/me/details", %{
        user: %{
          birth_date: @invalid_attrs[:birth_date],
        }
      })

    assert html_response(conn, 200) =~ "Update your user"

    unchanged_user = Repo.get(User, user.id)
    refute unchanged_user.birth_date == @invalid_attrs[:birth_date]
  end

  test "1.6 Authenticated user should not be able to update data for invalid phone number", %{conn: conn, user: user} do
    conn = setup_session(conn, user)

    conn = conn
      |> post("/me/details", %{
        user: %{
          phone_number: @invalid_attrs[:phone_number],
        }
      })

    assert html_response(conn, 200) =~ "Update your user"

    unchanged_user = Repo.get(User, user.id)
    refute unchanged_user.phone_number == @invalid_attrs[:phone_number]
  end



  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: "1234")
    conn = get(conn, redirected_to(conn))
    conn
  end
end
