defmodule PropTrackrWeb.PropertyNavigationTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite
  alias PropTrackr.Repo

  @password "password123"
  @valid_property %{
    title: "Test Property",
    description: "Small apartment with a beautiful view of the city center",
    type: "sell",
    property_type: "house",
    state: "available",
    location: "Test Location",
    room_count: 3,
    area: 100.0,
    floor: 2,
    floor_count: 5,
    price: 500000.0
  }

  setup do
    owner = %User{
      name: "Property",
      surname: "Owner",
      birth_date: ~D[2000-01-01],
      phone_number: "111",
      bio: "Hey",
      email: "property.owner@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    owner = Repo.insert!(owner)

    viewer = %User{
      name: "Property",
      surname: "Viewer",
      birth_date: ~D[2000-01-01],
      phone_number: "222",
      bio: "Hi",
      email: "property.viewer@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    viewer = Repo.insert!(viewer)

    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: owner.id
    }
    property = property
    |> Property.changeset(@valid_property)
    |> Repo.insert!()

    {:ok, %{owner: owner, viewer: viewer, property: property}}
  end

  describe "property navigation from different views" do
    test "clicking property in home listing shows details", %{conn: conn, property: property} do
      conn = get(conn, "/")
      assert html_response(conn, 200) =~ property.title
      assert html_response(conn, 200) =~ ~s|href="/properties/#{property.reference}"|

      conn = get(conn, "/properties/#{property.reference}")
      response = html_response(conn, 200)
      assert response =~ property.title
      assert response =~ property.description
    end

    test "clicking property in favorites shows details", %{conn: conn, viewer: viewer, property: property} do
      Repo.insert!(%Favorite{user_id: viewer.id, property_id: property.id})

      conn = conn |> setup_session(viewer)
      conn = get(conn, "/me/favorites")
      assert html_response(conn, 200) =~ property.title
      assert html_response(conn, 200) =~ ~s|href="/properties/#{property.reference}"|

      conn = get(conn, "/properties/#{property.reference}")
      response = html_response(conn, 200)
      assert response =~ property.title
      assert response =~ property.description
    end

    test "clicking property in owner profile shows details", %{conn: conn, owner: owner, property: property} do
      conn = get(conn, "/owners/#{owner.id}")
      assert html_response(conn, 200) =~ property.title
      assert html_response(conn, 200) =~ ~s|href="/properties/#{property.reference}"|

      conn = get(conn, "/properties/#{property.reference}")
      response = html_response(conn, 200)
      assert response =~ property.title
      assert response =~ property.description
    end

    test "clicking property in my properties shows details", %{conn: conn, owner: owner, property: property} do
      conn = conn |> setup_session(owner)
      conn = get(conn, "/me/properties")
      assert html_response(conn, 200) =~ property.title
      assert html_response(conn, 200) =~ ~s|href="/properties/#{property.reference}"|

      conn = get(conn, "/properties/#{property.reference}")
      response = html_response(conn, 200)
      assert response =~ property.title
      assert response =~ property.description
    end

    test "clicking non-existent property returns error", %{conn: conn} do
      conn = get(conn, "/properties/non-existent-reference")
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) == "Property not found."
    end
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
