defmodule PropTrackrWeb.RegisterControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Repo

  @password "1234"

  @valid_attrs %{
    name: "John",
    surname: "Doe",
    birth_date: ~D[1990-01-01],
    phone_number: "123456789",
    email: "john.doe@example.com",
    password: @password,
    confirm_password: @password,
    hashed_password: Bcrypt.hash_pwd_salt(@password)
  }

  @invalid_phone_attrs %{
    name: "Jane",
    surname: "Doe",
    birth_date: ~D[1990-01-01],
    phone_number: "123abc",
    email: "jane.doe@example.com",
    password: @password,
    confirm_password: @password,
    hashed_password: Bcrypt.hash_pwd_salt(@password)
  }

  @future_date_attrs %{
    name: "Michael",
    surname: "Smith",
    birth_date: ~D[2500-01-01],
    phone_number: "987654321",
    email: "michael.smith@example.com",
    password: @password,
    confirm_password: @password,
    hashed_password: Bcrypt.hash_pwd_salt(@password)
  }

  @mismatched_password_attrs %{
    name: "Jane",
    surname: "Doe",
    birth_date: ~D[1990-01-01],
    phone_number: "123456789",
    email: "jane.doe@example.com",
    password: @password,
    confirm_password: "mismatch_password",
    hashed_password: Bcrypt.hash_pwd_salt(@password)
  }

  setup %{conn: conn} do
    {:ok, conn: conn}
  end

  describe "user registration" do
    test "successful registration with valid attributes", %{conn: conn} do
      conn = post(conn, ~p"/register", user: @valid_attrs)

      # Assert redirect to home page
      assert redirected_to(conn) == ~p"/"

      # Assert flash message
      assert get_flash(conn, :info) == "User created successfully."

      # Assert user was saved in database
      user = Repo.get_by!(User, email: @valid_attrs.email)
      assert user.name == @valid_attrs.name
      assert user.surname == @valid_attrs.surname
      assert user.phone_number == @valid_attrs.phone_number
    end

    test "registration fails with invalid phone number", %{conn: conn} do
      conn = post(conn, ~p"/register", user: @invalid_phone_attrs)

      # Assert rendering registration form again
      assert html_response(conn, 200) =~ "Register"

      # Assert error message
      assert html_response(conn, 200) =~ "must be between 3 and 15 digits, without &#39;+&#39; sign"

      # Assert user was not saved
      assert Repo.get_by(User, email: @invalid_phone_attrs.email) == nil
    end

    test "registration fails with future birth date", %{conn: conn} do
      conn = post(conn, ~p"/register", user: @future_date_attrs)

      # Assert rendering registration form again
      assert html_response(conn, 200) =~ "Register"

      # Assert user was not saved
      assert Repo.get_by(User, email: @future_date_attrs.email) == nil
    end

    test "registration fails with existing email", %{conn: conn} do
      # First create a user
      {:ok, _existing_user} = %User{}
      |> User.changeset(@valid_attrs)
      |> Repo.insert()

      # Try to register with same email
      duplicate_email_attrs = %{@valid_attrs | name: "Jane", surname: "Smith"}
      conn = post(conn, ~p"/register", user: duplicate_email_attrs)

      # Assert rendering registration form again
      assert html_response(conn, 200) =~ "Register"

      # Assert error message
      assert html_response(conn, 200) =~ "An account with this email already exists"

      # Assert only one user exists with this email
      user = Repo.get_by!(User, email: @valid_attrs.email)
    end

    test "registration fails with mismatched passwords", %{conn: conn} do
      conn = post(conn, ~p"/register", user: @mismatched_password_attrs)

      # Assert rendering registration form again
      assert html_response(conn, 200) =~ "Register"

      # Assert error message
      assert html_response(conn, 200) =~ "Passwords do not match"

      # Assert user was not saved
      assert Repo.get_by(User, email: @mismatched_password_attrs.email) == nil
    end
  end

end
