defmodule PropTrackrWeb.SearchControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.NotInterested
  alias PropTrackr.Repo
  alias PropTrackr.Search
  import Ecto.Query, only: [from: 2]

  @password "1234"

  setup do
    user = %User{
      name: "Owner",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    user = Repo.insert!(user)

    another_user = %User{
      name: "Another user",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Yo",
      email: "another.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    another_user = Repo.insert!(another_user)

    properties = [
      %Property{
        title: "Rent property cheap",
        description: "This is a really cool property",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "London, UK",
        room_count: 1,
        area: 90.0,
        floor: 2,
        floor_count: 8,
        price: 500.0,
      },
      %Property{
        title: "Rent property cool",
        description: "Also a really cool property",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "London, UK",
        room_count: 3,
        area: 160.0,
        floor: 2,
        floor_count: 5,
        price: 6000.0,
      },
      %Property{
        title: "Rent property Tartu",
        description: "Also a really cool property",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "Tartu, Estonia",
        room_count: 2,
        area: 120.0,
        floor: 1,
        floor_count: 2,
        price: 800.0,
      },
      %Property{
        title: "Buy property Tartu",
        description: "Also a really cool property",
        type: :sell,
        property_type: :apartment,
        state: :available,
        location: "Tartu, Estonia",
        room_count: 2,
        area: 120.0,
        floor: 1,
        floor_count: 2,
        price: 800000.0,
      },
      %Property{
        title: "Buy property Tallinn",
        description: "Also a really cool property",
        type: :sell,
        property_type: :apartment,
        state: :available,
        location: "Tallinn, Estonia",
        room_count: 2,
        area: 120.0,
        floor: 1,
        floor_count: 2,
        price: 800000.0,
      }
    ]
    Enum.each(properties, fn property -> Repo.insert!(property) end)

    {:ok, %{user: user, another_user: another_user, properties: properties}}
  end

  test "3.2, 3.3 Searching with boundaries should show results within the boundaries for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    location = "Tartu, Estonia"
    shown_properties = properties |> Enum.filter(fn p -> p.location == location end)
    other_properties = properties |> Enum.reject(fn p -> p.location == location end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.2, 3.3 Searching with multiple areas should show results within the areas for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> Enum.member?(locations, p.location) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.2, 3.3 Searching with type filter rent should show proper results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :rent,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.type == :rent and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.2, 3.3 Searching with type filter sell should show proper results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :sell,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.type == :sell and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.2, 3.3 Searching outside of the price range should show no results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100000000.0,
      max_price: 1000000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    result = html_response(conn, 200)
    assert result =~ "No advertisements at the moment"
  end

  test "3.2, 3.3 Searching outside of the room range should show no results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 100,
      max_rooms: 1000,
    }})

    result = html_response(conn, 200)
    assert result =~ "No advertisements at the moment"
  end

  test "3.2, 3.3 Searching within correct price range should show proper results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    properties = Repo.all(Property)

    min_price = 100
    max_price = 800

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: min_price,
      max_price: max_price,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.price >= min_price and p.price <= max_price and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.2, 3.3 Searching within correct room count range should show proper results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    properties = Repo.all(Property)

    min_rooms = 1
    max_rooms = 2

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: min_rooms,
      max_rooms: max_rooms,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.room_count >= min_rooms and p.room_count <= max_rooms and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.2, 3.3 Searching without any areas should show all results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: [],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    result = html_response(conn, 200)
    assert result =~ "Search results"

    properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)
  end

  test "3.2, 3.3 Searching with non-existent with type but existing location should show no results for authenticated user", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    conn = post(conn, ~p"/search", %{search: %{
      type: :sell,
      location: "UK",
      areas: ["London"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    result = html_response(conn, 200)
    assert result =~ "No advertisements at the moment"
  end

  test "Unauthenticated user should not be able to use the advanced search feature", %{ conn: conn, user: _user, properties: _properties } do
    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    assert redirected_to(conn) == "/login"
    conn = get conn, redirected_to(conn)
    assert get_flash(conn, :error) =~ ~r/You are not logged in/
  end

  test "4.4 The last 5 searches of authenticated user should be saved", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    searches = Repo.all(from s in Search, where: s.user_id == ^user.id, order_by: [asc: s.inserted_at])
    assert length(searches) == 0

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Jõgeva"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = get conn, "/search"

    result = html_response(conn, 200)
    assert result =~ ~r/Previous searches/

    searches = Repo.all(from s in Search, where: s.user_id == ^user.id, order_by: [asc: s.inserted_at])
    assert length(searches) == 5
  end

  test "4.4 Authenticated user should be able to see a previous search by clicking it", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = get conn, "/search"

    result = html_response(conn, 200)
    assert result =~ ~r/Previous searches/

    searches = Repo.all(from s in Search, where: s.user_id == ^user.id, order_by: [asc: s.inserted_at])
    assert length(searches) == 1

    conn = get conn, "/search/#{Enum.at(searches, 0).id}"

    result = html_response(conn, 200)
    assert result =~ ~r/Search results/
  end

  test "4.4 Unauthenticated user should be redirected to login page when trying to view a saved search page", %{ conn: conn } do
    conn = get conn, "/search/1"

    assert redirected_to(conn) == "/login"
    conn = get conn, redirected_to(conn)

    assert get_flash(conn, :error) =~ ~r/You are not logged in/
  end

  test "4.4 Authenticated user should not see another user's search when clicking on it", %{ conn: conn, user: user, another_user: another_user, properties: _properties } do
    conn = conn |> setup_session(user)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    conn = get conn, "/search"

    result = html_response(conn, 200)
    assert result =~ ~r/Previous searches/

    searches = Repo.all(from s in Search, where: s.user_id == ^user.id, order_by: [asc: s.inserted_at])
    assert length(searches) == 1

    conn = get conn, "/search/#{Enum.at(searches, 0).id}"

    result = html_response(conn, 200)
    assert result =~ ~r/Search results/

    conn = conn |> setup_session(another_user)

    conn = get conn, "/search"

    result = html_response(conn, 200)
    refute result =~ ~r/Previous searches/

    # Note: This should be 0 since the user does not have any searches
    searches = Repo.all(from s in Search, where: s.user_id == ^another_user.id, order_by: [asc: s.inserted_at])
    assert length(searches) == 0

    # Note: Now checking if the user can see another user's search
    searches = Repo.all(from s in Search, order_by: [asc: s.inserted_at])
    assert length(searches) == 1

    conn = get conn, "/search/#{Enum.at(searches, 0).id}"

    assert redirected_to(conn) == "/search"
    conn = get conn, redirected_to(conn)
    assert get_flash(conn, :error) =~ ~r/Search not found/
  end

  test "3.1 Unauthenticated user should be able to do a simple search", %{ conn: conn, user: user, properties: _properties } do
    term = "Tartu"
    conn = post(conn, ~p"/search?simple=1", %{search_term: term})
    result = html_response(conn, 200)

    properties = Repo.all(Property)
    show_filter = fn p ->
      p.title =~ ~r/#{term}/ or p.description =~ ~r/#{term}/ or p.location =~ ~r/#{term}/
    end
    shown_properties = properties |> Enum.filter(show_filter)
    not_shown_properties = properties |> Enum.reject(show_filter)

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    not_shown_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.1 Authenticated user should be able to do a simple search", %{ conn: conn, user: user, properties: _properties } do
    conn = conn |> setup_session(user)

    term = "Tartu"
    conn = post(conn, ~p"/search?simple=1", %{search_term: term})
    result = html_response(conn, 200)

    properties = Repo.all(Property)
    show_filter = fn p ->
      p.title =~ ~r/#{term}/ or p.description =~ ~r/#{term}/ or p.location =~ ~r/#{term}/
    end
    shown_properties = properties |> Enum.filter(show_filter)
    not_shown_properties = properties |> Enum.reject(show_filter)

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    not_shown_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "3.1 User should receive an error if the search term is missing or too short", %{ conn: conn, user: user, properties: _properties } do
    term = ""
    conn = post(conn, ~p"/search?simple=1", %{search_term: term})
    assert redirected_to(conn) == "/"
    conn = get conn, redirected_to(conn)
    assert get_flash(conn, :error) =~ ~r/Please insert a search term/

    term = "12"
    conn = post(conn, ~p"/search?simple=1", %{search_term: term})
    assert redirected_to(conn) == "/"
    conn = get conn, redirected_to(conn)
    assert get_flash(conn, :error) =~ ~r/Search term should be at least 3 characters/
  end

  test "3.7 Search order should be correct", %{ conn: conn, user: user, another_user: another_user, properties: _properties } do
    Repo.delete_all(Property)

    ads = [
      %Property{
        title: "Available property and not interested",
        description: "Renting this really really house",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "London, UK",
        room_count: 3,
        area: 100.0,
        floor: 2,
        floor_count: 5,
        price: 500000.0,
      },
      %Property{
        title: "Unavailable property and interested",
        description: "Renting this really really house",
        type: :rent,
        property_type: :apartment,
        state: :reserved,
        location: "London, UK",
        room_count: 3,
        area: 100.0,
        floor: 2,
        floor_count: 5,
        price: 500000.0,
      },
      %Property{
        title: "Available property and interested",
        description: "Renting this really really house",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "London, UK",
        room_count: 3,
        area: 100.0,
        floor: 2,
        floor_count: 5,
        price: 500000.0,
      },
      %Property{
        title: "Unavailable property and not interested",
        description: "Renting this really really house",
        type: :rent,
        property_type: :apartment,
        state: :reserved,
        location: "London, UK",
        room_count: 3,
        area: 100.0,
        floor: 2,
        floor_count: 5,
        price: 500000.0,
      }
    ]
    Enum.each(ads, fn ad -> Repo.insert!(ad) end)

    Repo.insert!(%NotInterested{
      user_id: user.id,
      property_id: Repo.get_by(Property, title: "Available property and not interested").id
    })
    Repo.insert!(%NotInterested{
      user_id: user.id,
      property_id: Repo.get_by(Property, title: "Unavailable property and not interested").id
    })

    conn = conn |> setup_session(user)

    # Note: This does not test the controller per-se, it's easier to test the query here

    search = %{
      type: :any,
      location: "UK",
      areas: ["London"],
      min_price: 50.0,
      max_price: 100000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }
    changeset = Search.changeset(%Search{}, search)
    query = Search.create_query(changeset.changes, user)
    properties = query |> Repo.all()

    correct_titles = [
      "Available property and interested",
      "Unavailable property and interested",
      "Available property and not interested",
      "Unavailable property and not interested"
    ]

    property_titles = Enum.map(properties, fn p -> p.title end)
    assert correct_titles == property_titles
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end

end
