defmodule PropTrackrWeb.SecurityTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Repo

  @password "password"
  @valid_user %{
    name: "John",
    surname: "Doe",
    birth_date: ~D[1990-01-02],
    phone_number: "123456789",
    email: "john.doe@gmail.com",
    password: "test123",
    confirm_password: "test123"
  }

  describe "password encryption" do
    test "password is encrypted when user registers", %{conn: conn} do
      conn = post(conn, "/register", user: @valid_user)
      assert redirected_to(conn) == "/"

      user = Repo.get_by(User, email: @valid_user.email)
      assert user.hashed_password != @valid_user.password
      assert String.starts_with?(user.hashed_password, "$2b$")

      conn = get(conn, redirected_to(conn))
      assert get_flash(conn, :info) =~ "User created successfully"
    end
  end

  describe "unauthorized property access" do
    setup do
      owner = %User{
        name: "First",
        surname: "User",
        birth_date: ~D[2000-01-01],
        phone_number: "111",
        email: "first.user@gmail.com",
        password: @password,
        confirm_password: @password,
        hashed_password: Bcrypt.hash_pwd_salt(@password)
      }
      owner = Repo.insert!(owner)

      other_user = %User{
        name: "Second",
        surname: "User",
        birth_date: ~D[2000-01-01],
        phone_number: "222",
        email: "second.user@gmail.com",
        password: @password,
        confirm_password: @password,
        hashed_password: Bcrypt.hash_pwd_salt(@password)
      }
      other_user = Repo.insert!(other_user)

      property = %Property{
        reference: Ecto.UUID.generate(),
        title: "Modern Apartment",
        description: "Great apartment with nice view",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "Tartu, Estonia",
        room_count: 2,
        area: 50.0,
        floor: 3,
        floor_count: 5,
        price: 500.0,
        user_id: owner.id
      }
      property = Repo.insert!(property)

      {:ok, %{owner: owner, other_user: other_user, property: property}}
    end

    test "non-owner cannot access property edit page", %{conn: conn, other_user: user, property: property} do
      conn = conn |> setup_session(user)
      conn = get(conn, ~p"/properties/#{property.reference}/edit")

      assert redirected_to(conn) == ~p"/properties/#{property.reference}"
      assert get_flash(conn, :error) == "You don't have permission to edit this property."
    end
  end

  describe "protected pages access" do
    test "unauthenticated user cannot access protected pages", %{conn: conn} do
      protected_paths = [
        "/me/properties",
        "/me/favorites",
        "/me",
        "/me/password",
        "/me/details"
      ]

      Enum.each(protected_paths, fn path ->
        conn = get(conn, path)
        assert redirected_to(conn) == "/login"
        assert get_flash(conn, :error) == "You are not logged in!"
      end)
    end

    test "logged out user cannot access protected pages", %{conn: conn} do
      user = %User{
        name: "First",
        surname: "User",
        birth_date: ~D[2000-01-01],
        phone_number: "111",
        email: "first.user@gmail.com",
        password: @password,
        confirm_password: @password,
        hashed_password: Bcrypt.hash_pwd_salt(@password)
      }
      Repo.insert!(user)

      conn = conn |> setup_session(user)
      conn = get(conn, ~p"/logout")
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "You are successfully logged out"

      conn = get(conn, "/me/properties")
      assert redirected_to(conn) == "/login"
      assert get_flash(conn, :error) == "You are not logged in!"
    end
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: @password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
