defmodule PropTrackrWeb.ThemeControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Repo

  @password "1234"

  setup do
    user = %User{
      name: "Test",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password),
      is_dark_mode: false
    }
    user = Repo.insert!(user)

    other_user = %User{
      name: "Other",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "111",
      bio: "Hey",
      email: "other.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password),
      is_dark_mode: true
    }
    other_user = Repo.insert!(other_user)

    {:ok, %{user: user, other_user: other_user}}
  end

  test "authenticated user can enable dark mode", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/update_theme", %{"is_dark_mode" => true})

    assert json_response(conn, 200) == %{
      "success" => true,
      "is_dark_mode" => true
    }

    updated_user = Repo.get!(User, user.id)
    assert updated_user.is_dark_mode == true
  end

  test "authenticated user can disable dark mode", %{conn: conn, other_user: user} do
    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/update_theme", %{"is_dark_mode" => false})

    assert json_response(conn, 200) == %{
      "success" => true,
      "is_dark_mode" => false
    }

    updated_user = Repo.get!(User, user.id)
    assert updated_user.is_dark_mode == false
  end

  test "unauthenticated user cannot update theme", %{conn: conn} do
    conn = post(conn, ~p"/api/update_theme", %{"is_dark_mode" => true})

    assert json_response(conn, 401) == %{
      "error" => "User not authenticated"
    }
  end

  test "theme preference persists in session", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/update_theme", %{"is_dark_mode" => true})

    assert get_session(conn, :user_theme) == true

    updated_user = Repo.get!(User, user.id)
    assert updated_user.is_dark_mode == true
  end

  test "user can toggle theme multiple times", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)

    # Enable dark mode
    conn = post(conn, ~p"/api/update_theme", %{"is_dark_mode" => true})
    assert json_response(conn, 200) == %{"success" => true, "is_dark_mode" => true}

    # Disable dark mode
    conn = post(conn, ~p"/api/update_theme", %{"is_dark_mode" => false})
    assert json_response(conn, 200) == %{"success" => true, "is_dark_mode" => false}

    # Enable dark mode again
    conn = post(conn, ~p"/api/update_theme", %{"is_dark_mode" => true})
    assert json_response(conn, 200) == %{"success" => true, "is_dark_mode" => true}

    updated_user = Repo.get!(User, user.id)
    assert updated_user.is_dark_mode == true
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
