defmodule PropTrackrWeb.UserDeleteControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Repo

  @valid_attrs %{email: "test.user@gmail.com", password: "1234"}
  @password "1234"

  setup do
    user = %User{
      name: "Test",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    user = Repo.insert!(user)

    {:ok, %{user: user}}
  end

  test "Authenticated user should be able to delete their account and be redirected to the homepage and logged out and data deleted", %{conn: conn, user: user} do
    # Log in the user
    conn = setup_session(conn, user)

    # Ensure the user is logged in
    assert get_session(conn, :user_id) == user.id

    # Perform delete with the user ID in the route
    conn = delete(conn, "/me/delete/#{user.id}")

    # Verify redirection to homepage
    assert redirected_to(conn) == "/"

    # Follow the redirection
    conn = get(conn, redirected_to(conn))
    assert html_response(conn, 200)

    # Ensure the response contains the correct flash message
    assert get_flash(conn, :info) =~ "Your account has been deleted successfully."

    # Verify the user is logged out
    assert get_session(conn, :user_id) == nil

    # Confirm the user's data is removed from the database
    assert Repo.get(User, user.id) == nil
  end

  test "Unauthenticated user cannot access the delete page", %{conn: conn} do
    # Attempt to access the delete account page
    conn = get(conn, "/me/delete")

    # Verify redirection to homepage
    assert redirected_to(conn) == "/login"

    # Follow the redirection
    conn = get(conn, redirected_to(conn))

    assert html_response(conn, 200)

  end

  test "Deleting user should delete all associated properties", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(%{
      title: "Test Property",
      description: "Test DescriptionTest DescriptionTest Description",
      type: "rent",
      property_type: "apartment",
      location: "Test Location",
      room_count: 2,
      area: 50.0,
      floor: 1,
      floor_count: 4,
      price: 500.0
    })
    |> Repo.insert!()

    conn = setup_session(conn, user)
    conn = delete(conn, "/me/delete/#{user.id}")

    assert redirected_to(conn) == "/"
    assert get_flash(conn, :info) =~ "Your account has been deleted successfully."
    assert Repo.get(User, user.id) == nil

    assert Repo.get(Property, property.id) == nil
   end


  defp setup_session(conn, user) do
    conn = conn |> post("/login", @valid_attrs)
    conn = get conn, redirected_to(conn)
    conn
  end

end
